# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from google.oauth2 import service_account


class ConstructableFromServiceAccount:
    @classmethod
    def from_service_account_file(cls, filename, **kwargs):
        f"""Creates an instance of this client using the provided credentials file.
        Args:
            filename (str): The path to the service account private key json
                file.
            kwargs: Additional arguments to pass to the constructor.
        Returns:
            A {cls.__name__}.
        """
        credentials = service_account.Credentials.from_service_account_file(filename)
        return cls(credentials=credentials, **kwargs)

    from_service_account_json = from_service_account_file



# -*- coding: utf-8 -*- #
# Copyright 2016 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""The configs variables set command."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import sys

from apitools.base.py import exceptions as apitools_exceptions

from googlecloudsdk.api_lib.runtime_config import util
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.runtime_config import flags
from googlecloudsdk.core import log
from googlecloudsdk.core.util import http_encoding


class Set(base.CreateCommand):
  """Create or update variable resources.

  This command creates or updates a variable resource, setting its value to
  the specified string or file contents.
  """

  detailed_help = {
      'EXAMPLES': """\
          To create or update a variable named "my-var", run:

            $ {command} --config-name=my-config my-var "my value"

          To create or update a variable with a hierarchical name, such as
          "results/task1", run:

            $ {command} --config-name=my-config results/task1 "my value"

          To create a variable, but fail if it already exists, run:

            $ {command} --config-name=my-config my-var "my-value" --fail-if-present

          To update a variable, but fail if it does not exist, run:

            $ {command} --config-name=my-config my-var "my-value" --fail-if-absent

          It is possible to provide --is-text flag if the value contains only
          text (UTF-8 encoded). This affects how the variable is transmitted on
          the wire and requires less quota on the backend.

            $ {command} --config-name=my-config --is-text my-var "my value"

          If the variable's value parameter is not specified, the value will be
          read from standard input. This allows setting variables to large values
          or values that contain non-printable characters. The variable value
          will be automatically base64-encoded. For example, to set a variable's
          value to the contents of a file, run:

            $ cat my-file | {command} --config-name my-config my-var
          """,
  }

  @staticmethod
  def Args(parser):
    """Args is called by calliope to gather arguments for this command.

    Args:
      parser: An argparse parser that you can use to add arguments that go
          on the command line after this command. Positional arguments are
          allowed.
    """
    flags.AddRequiredConfigFlag(parser)

    fail_group = parser.add_mutually_exclusive_group()
    fail_group.add_argument(
        '--fail-if-present',
        help='Fail if a variable with the specified name already exists.',
        action='store_true')
    fail_group.add_argument(
        '--fail-if-absent',
        help='Fail if a variable with the specified name does not exist.',
        action='store_true')

    parser.add_argument('name', help='The variable name.')
    parser.add_argument(
        'value',
        nargs='?',
        default=None,
        help=(
            'The variable value. If absent, the value will be read from stdin. '
            'The value is automatically base64-encoded, '
            'unless --is-text flag is set.'))

    parser.add_argument('--is-text',
                        default=False,
                        required=False,
                        action='store_true',
                        help=('If True, send and store the value as text. Can '
                              'be used if the value contains only text '
                              '(UTF-8 encoded). This affects how the variable '
                              'is transmitted on the wire and requires less '
                              'quota on the backend.'))

  def Run(self, args):
    """Run 'runtime-configs variables set'.

    Args:
      args: argparse.Namespace, The arguments that this command was invoked
          with.

    Returns:
      The new variable.

    Raises:
      HttpException: An http error response was received while executing api
          request.
    """
    var_resource = util.ParseVariableName(args.name, args)
    if args.value is None:
      log.status.Print('No value argument specified; reading value from stdin.')
      value = sys.stdin.read()
    else:
      value = args.value

    if args.fail_if_absent:
      # Update case
      return self._Update(args, var_resource, value)
    else:
      # Either create or create-or-update
      try:
        return self._Create(args, var_resource, value)
      except apitools_exceptions.HttpConflictError:
        # If --fail-if-present was not specified, and we got an
        # Already Exists error, try updating instead.
        if not args.fail_if_present:
          return self._Update(args, var_resource, value)

        # If --fail-if-present was specified re-raise the error.
        raise

  def _Create(self, args, var_resource, value):
    variable_client = util.VariableClient()
    messages = util.Messages()

    project = var_resource.projectsId
    config = var_resource.configsId

    result = variable_client.Create(
        messages.RuntimeconfigProjectsConfigsVariablesCreateRequest(
            parent=util.ConfigPath(project, config),
            variable=messages.Variable(
                name=var_resource.RelativeName(),
                value=http_encoding.Encode(value) if not args.is_text else None,
                text=value if args.is_text else None,
            )
        )
    )

    log.CreatedResource(var_resource)
    return util.FormatVariable(result)

  def _Update(self, args, var_resource, value):
    variable_client = util.VariableClient()
    messages = util.Messages()

    result = variable_client.Update(
        messages.Variable(
            name=var_resource.RelativeName(),
            value=http_encoding.Encode(value) if not args.is_text else None,
            text=value if args.is_text else None,
        )
    )

    log.UpdatedResource(var_resource)
    return util.FormatVariable(result)



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from abc import ABC, abstractmethod
from typing import Callable

from google.pubsub_v1 import PubsubMessage
from overrides import overrides

from google.cloud.pubsublite_v1 import SequencedMessage


class MessageTransformer(ABC):
    """
    A MessageTransformer turns Pub/Sub Lite message protos into Pub/Sub message protos.
    """

    @abstractmethod
    def transform(self, source: SequencedMessage) -> PubsubMessage:
        """Transform a SequencedMessage to a PubsubMessage.

        Args:
          source: The message to transform.

        Raises:
          GoogleAPICallError: To fail the client if raised inline.
        """
        pass

    @staticmethod
    def of_callable(transformer: Callable[[SequencedMessage], PubsubMessage]):
        class CallableTransformer(MessageTransformer):
            @overrides
            def transform(self, source: SequencedMessage) -> PubsubMessage:
                return transformer(source)

        return CallableTransformer()



# coding: utf-8
"""
    Kubernetes

    No description provided (generated by Swagger Codegen
    https://github.com/swagger-api/swagger-codegen)

    OpenAPI spec version: v1.14.4

    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

import os
import re
import json
import mimetypes
import tempfile
from multiprocessing.pool import ThreadPool

from datetime import date, datetime

# python 2 and python 3 compatibility library
from six import PY3, integer_types, iteritems, text_type
from six.moves.urllib.parse import quote

from . import models
from .configuration import Configuration
from .rest import ApiException, RESTClientObject


class ApiClient(object):
  """
    Generic API client for Swagger client library builds.

    Swagger generic API client. This client handles the client-
    server communication, and is invariant across implementations. Specifics of
    the methods and models for each application are generated from the Swagger
    templates.

    NOTE: This class is auto generated by the swagger code generator program.
    Ref: https://github.com/swagger-api/swagger-codegen
    Do not edit the class manually.

    :param host: The base path for the server to call.
    :param header_name: a header to pass when making calls to the API.
    :param header_value: a header value to pass when making calls to the API.
    """

  PRIMITIVE_TYPES = (float, bool, bytes, text_type) + integer_types
  NATIVE_TYPES_MAPPING = {
      'int': int,
      'long': int if PY3 else long,
      'float': float,
      'str': str,
      'bool': bool,
      'date': date,
      'datetime': datetime,
      'object': object,
  }
  _pool = None

  def __init__(self,
               configuration=None,
               header_name=None,
               header_value=None,
               cookie=None,
               pool_threads=None):
    if configuration is None:
      configuration = Configuration()
    self.configuration = configuration
    self.pool_threads = pool_threads

    self.rest_client = RESTClientObject(configuration)
    self.default_headers = {}
    if header_name is not None:
      self.default_headers[header_name] = header_value
    self.cookie = cookie
    # Set default User-Agent.
    self.user_agent = 'Swagger-Codegen/10.0.0-snapshot/python'

  def __del__(self):
    if self._pool:
      self._pool.close()
      self._pool.join()
      self._pool = None

  @property
  def pool(self):
    """Create thread pool on first request

         avoids instantiating unused threadpool for blocking clients.
        """
    if self._pool is None:
      self._pool = ThreadPool(self.pool_threads)
    return self._pool

  @property
  def user_agent(self):
    """
        Gets user agent.
        """
    return self.default_headers['User-Agent']

  @user_agent.setter
  def user_agent(self, value):
    """
        Sets user agent.
        """
    self.default_headers['User-Agent'] = value

  def set_default_header(self, header_name, header_value):
    self.default_headers[header_name] = header_value

  def __call_api(self,
                 resource_path,
                 method,
                 path_params=None,
                 query_params=None,
                 header_params=None,
                 body=None,
                 post_params=None,
                 files=None,
                 response_type=None,
                 auth_settings=None,
                 _return_http_data_only=None,
                 collection_formats=None,
                 _preload_content=True,
                 _request_timeout=None):

    config = self.configuration

    # header parameters
    header_params = header_params or {}
    header_params.update(self.default_headers)
    if self.cookie:
      header_params['Cookie'] = self.cookie
    if header_params:
      header_params = self.sanitize_for_serialization(header_params)
      header_params = dict(
          self.parameters_to_tuples(header_params, collection_formats))

    # path parameters
    if path_params:
      path_params = self.sanitize_for_serialization(path_params)
      path_params = self.parameters_to_tuples(path_params, collection_formats)
      for k, v in path_params:
        # specified safe chars, encode everything
        resource_path = resource_path.replace(
            '{%s}' % k, quote(str(v), safe=config.safe_chars_for_path_param))

    # query parameters
    if query_params:
      query_params = self.sanitize_for_serialization(query_params)
      query_params = self.parameters_to_tuples(query_params, collection_formats)

    # post parameters
    if post_params or files:
      post_params = self.prepare_post_parameters(post_params, files)
      post_params = self.sanitize_for_serialization(post_params)
      post_params = self.parameters_to_tuples(post_params, collection_formats)

    # auth setting
    self.update_params_for_auth(header_params, query_params, auth_settings)

    # body
    if body:
      body = self.sanitize_for_serialization(body)

    # request url
    url = self.configuration.host + resource_path

    # perform request and return response
    response_data = self.request(
        method,
        url,
        query_params=query_params,
        headers=header_params,
        post_params=post_params,
        body=body,
        _preload_content=_preload_content,
        _request_timeout=_request_timeout)

    self.last_response = response_data

    return_data = response_data
    if _preload_content:
      # deserialize response data
      if response_type:
        return_data = self.deserialize(response_data, response_type)
      else:
        return_data = None

    if _return_http_data_only:
      return (return_data)
    else:
      return (return_data, response_data.status, response_data.getheaders())

  def sanitize_for_serialization(self, obj):
    """
        Builds a JSON POST object.

        If obj is None, return None.
        If obj is str, int, long, float, bool, return directly.
        If obj is datetime.datetime, datetime.date
            convert to string in iso8601 format.
        If obj is list, sanitize each element in the list.
        If obj is dict, return the dict.
        If obj is swagger model, return the properties dict.

        :param obj: The data to serialize.
        :return: The serialized form of data.
        """
    if obj is None:
      return None
    elif isinstance(obj, self.PRIMITIVE_TYPES):
      return obj
    elif isinstance(obj, list):
      return [self.sanitize_for_serialization(sub_obj) for sub_obj in obj]
    elif isinstance(obj, tuple):
      return tuple(self.sanitize_for_serialization(sub_obj) for sub_obj in obj)
    elif isinstance(obj, (datetime, date)):
      return obj.isoformat()

    if isinstance(obj, dict):
      obj_dict = obj
    else:
      # Convert model obj to dict except
      # attributes `swagger_types`, `attribute_map`
      # and attributes which value is not None.
      # Convert attribute name to json key in
      # model definition for request.
      obj_dict = {
          obj.attribute_map[attr]: getattr(obj, attr)
          for attr, _ in iteritems(obj.swagger_types)
          if getattr(obj, attr) is not None
      }

    return {
        key: self.sanitize_for_serialization(val)
        for key, val in iteritems(obj_dict)
    }

  def deserialize(self, response, response_type):
    """
        Deserializes response into an object.

        :param response: RESTResponse object to be deserialized.
        :param response_type: class literal for
            deserialized object, or string of class name.

        :return: deserialized object.
        """
    # handle file downloading
    # save response body into a tmp file and return the instance
    if response_type == 'file':
      return self.__deserialize_file(response)

    # fetch data from response object
    try:
      data = json.loads(response.data)
    except ValueError:
      data = response.data

    return self.__deserialize(data, response_type)

  def __deserialize(self, data, klass):
    """
        Deserializes dict, list, str into an object.

        :param data: dict, list or str.
        :param klass: class literal, or string of class name.

        :return: object.
        """
    if data is None:
      return None

    if type(klass) == str:
      if klass.startswith('list['):
        sub_kls = re.match('list\[(.*)\]', klass).group(1)
        return [self.__deserialize(sub_data, sub_kls) for sub_data in data]

      if klass.startswith('dict('):
        sub_kls = re.match('dict\(([^,]*), (.*)\)', klass).group(2)
        return {k: self.__deserialize(v, sub_kls) for k, v in iteritems(data)}

      # convert str to class
      if klass in self.NATIVE_TYPES_MAPPING:
        klass = self.NATIVE_TYPES_MAPPING[klass]
      else:
        klass = getattr(models, klass)

    if klass in self.PRIMITIVE_TYPES:
      return self.__deserialize_primitive(data, klass)
    elif klass == object:
      return self.__deserialize_object(data)
    elif klass == date:
      return self.__deserialize_date(data)
    elif klass == datetime:
      return self.__deserialize_datatime(data)
    else:
      return self.__deserialize_model(data, klass)

  def call_api(self,
               resource_path,
               method,
               path_params=None,
               query_params=None,
               header_params=None,
               body=None,
               post_params=None,
               files=None,
               response_type=None,
               auth_settings=None,
               async_req=None,
               _return_http_data_only=None,
               collection_formats=None,
               _preload_content=True,
               _request_timeout=None):
    """
        Makes the HTTP request (synchronous) and return the deserialized data.
        To make an async request, set the async parameter.

        :param resource_path: Path to method endpoint.
        :param method: Method to call.
        :param path_params: Path parameters in the url.
        :param query_params: Query parameters in the url.
        :param header_params: Header parameters to be
            placed in the request header.
        :param body: Request body.
        :param post_params dict: Request post form parameters,
            for `application/x-www-form-urlencoded`, `multipart/form-data`.
        :param auth_settings list: Auth Settings names for the request.
        :param response: Response data type.
        :param files dict: key -> filename, value -> filepath,
            for `multipart/form-data`.
        :param async_req bool: execute request asynchronously
        :param _return_http_data_only: response data without head status code
        and headers
        :param collection_formats: dict of collection formats for path, query,
            header, and post parameters.
        :param _preload_content: if False, the urllib3.HTTPResponse object will
        be returned without
                                 reading/decoding response data. Default is
                                 True.
        :param _request_timeout: timeout setting for this request. If one number
        provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return:
            If async parameter is True,
            the request will be called asynchronously.
            The method will return the request thread.
            If parameter async is False or missing,
            then the method will return the response directly.
        """
    if not async_req:
      return self.__call_api(resource_path, method, path_params, query_params,
                             header_params, body, post_params, files,
                             response_type, auth_settings,
                             _return_http_data_only, collection_formats,
                             _preload_content, _request_timeout)
    else:
      thread = self.pool.apply_async(
          self.__call_api,
          (resource_path, method, path_params, query_params, header_params,
           body, post_params, files, response_type, auth_settings,
           _return_http_data_only, collection_formats, _preload_content,
           _request_timeout))
    return thread

  def request(self,
              method,
              url,
              query_params=None,
              headers=None,
              post_params=None,
              body=None,
              _preload_content=True,
              _request_timeout=None):
    """
        Makes the HTTP request using RESTClient.
        """
    if method == 'GET':
      return self.rest_client.GET(
          url,
          query_params=query_params,
          _preload_content=_preload_content,
          _request_timeout=_request_timeout,
          headers=headers)
    elif method == 'HEAD':
      return self.rest_client.HEAD(
          url,
          query_params=query_params,
          _preload_content=_preload_content,
          _request_timeout=_request_timeout,
          headers=headers)
    elif method == 'OPTIONS':
      return self.rest_client.OPTIONS(
          url,
          query_params=query_params,
          headers=headers,
          post_params=post_params,
          _preload_content=_preload_content,
          _request_timeout=_request_timeout,
          body=body)
    elif method == 'POST':
      return self.rest_client.POST(
          url,
          query_params=query_params,
          headers=headers,
          post_params=post_params,
          _preload_content=_preload_content,
          _request_timeout=_request_timeout,
          body=body)
    elif method == 'PUT':
      return self.rest_client.PUT(
          url,
          query_params=query_params,
          headers=headers,
          post_params=post_params,
          _preload_content=_preload_content,
          _request_timeout=_request_timeout,
          body=body)
    elif method == 'PATCH':
      return self.rest_client.PATCH(
          url,
          query_params=query_params,
          headers=headers,
          post_params=post_params,
          _preload_content=_preload_content,
          _request_timeout=_request_timeout,
          body=body)
    elif method == 'DELETE':
      return self.rest_client.DELETE(
          url,
          query_params=query_params,
          headers=headers,
          _preload_content=_preload_content,
          _request_timeout=_request_timeout,
          body=body)
    else:
      raise ValueError('http method must be `GET`, `HEAD`, `OPTIONS`,'
                       ' `POST`, `PATCH`, `PUT` or `DELETE`.')

  def parameters_to_tuples(self, params, collection_formats):
    """
        Get parameters as list of tuples, formatting collections.

        :param params: Parameters as dict or list of two-tuples
        :param dict collection_formats: Parameter collection formats
        :return: Parameters as list of tuples, collections formatted
        """
    new_params = []
    if collection_formats is None:
      collection_formats = {}
    for k, v in iteritems(params) if isinstance(params, dict) else params:
      if k in collection_formats:
        collection_format = collection_formats[k]
        if collection_format == 'multi':
          new_params.extend((k, value) for value in v)
        else:
          if collection_format == 'ssv':
            delimiter = ' '
          elif collection_format == 'tsv':
            delimiter = '\t'
          elif collection_format == 'pipes':
            delimiter = '|'
          else:  # csv is the default
            delimiter = ','
          new_params.append((k, delimiter.join(str(value) for value in v)))
      else:
        new_params.append((k, v))
    return new_params

  def prepare_post_parameters(self, post_params=None, files=None):
    """
        Builds form parameters.

        :param post_params: Normal form parameters.
        :param files: File parameters.
        :return: Form parameters with files.
        """
    params = []

    if post_params:
      params = post_params

    if files:
      for k, v in iteritems(files):
        if not v:
          continue
        file_names = v if type(v) is list else [v]
        for n in file_names:
          with open(n, 'rb') as f:
            filename = os.path.basename(f.name)
            filedata = f.read()
            mimetype = mimetypes.\
                guess_type(filename)[0] or 'application/octet-stream'
            params.append(tuple([k, tuple([filename, filedata, mimetype])]))

    return params

  def select_header_accept(self, accepts):
    """
        Returns `Accept` based on an array of accepts provided.

        :param accepts: List of headers.
        :return: Accept (e.g. application/json).
        """
    if not accepts:
      return

    accepts = [x.lower() for x in accepts]

    if 'application/json' in accepts:
      return 'application/json'
    else:
      return ', '.join(accepts)

  def select_header_content_type(self, content_types):
    """
        Returns `Content-Type` based on an array of content_types provided.

        :param content_types: List of content-types.
        :return: Content-Type (e.g. application/json).
        """
    if not content_types:
      return 'application/json'

    content_types = [x.lower() for x in content_types]

    if 'application/json' in content_types or '*/*' in content_types:
      return 'application/json'
    else:
      return content_types[0]

  def update_params_for_auth(self, headers, querys, auth_settings):
    """
        Updates header and query params based on authentication setting.

        :param headers: Header parameters dict to be updated.
        :param querys: Query parameters tuple list to be updated.
        :param auth_settings: Authentication setting identifiers list.
        """
    if not auth_settings:
      return

    for auth in auth_settings:
      auth_setting = self.configuration.auth_settings().get(auth)
      if auth_setting:
        if not auth_setting['value']:
          continue
        elif auth_setting['in'] == 'header':
          headers[auth_setting['key']] = auth_setting['value']
        elif auth_setting['in'] == 'query':
          querys.append((auth_setting['key'], auth_setting['value']))
        else:
          raise ValueError(
              'Authentication token must be in `query` or `header`')

  def __deserialize_file(self, response):
    """
        Saves response body into a file in a temporary folder,
        using the filename from the `Content-Disposition` header if provided.

        :param response:  RESTResponse.
        :return: file path.
        """
    fd, path = tempfile.mkstemp(dir=self.configuration.temp_folder_path)
    os.close(fd)
    os.remove(path)

    content_disposition = response.getheader('Content-Disposition')
    if content_disposition:
      filename = re.\
          search(r'filename=[\'"]?([^\'"\s]+)[\'"]?', content_disposition).\
          group(1)
      path = os.path.join(os.path.dirname(path), filename)

    with open(path, 'w') as f:
      f.write(response.data)

    return path

  def __deserialize_primitive(self, data, klass):
    """
        Deserializes string to primitive type.

        :param data: str.
        :param klass: class literal.

        :return: int, long, float, str, bool.
        """
    try:
      return klass(data)
    except UnicodeEncodeError:
      return unicode(data)
    except TypeError:
      return data

  def __deserialize_object(self, value):
    """
        Return a original value.

        :return: object.
        """
    return value

  def __deserialize_date(self, string):
    """
        Deserializes string to date.

        :param string: str.
        :return: date.
        """
    try:
      from dateutil.parser import parse
      return parse(string).date()
    except ImportError:
      return string
    except ValueError:
      raise ApiException(
          status=0,
          reason='Failed to parse `{0}` into a date object'.format(string))

  def __deserialize_datatime(self, string):
    """
        Deserializes string to datetime.

        The string should be in iso8601 datetime format.

        :param string: str.
        :return: datetime.
        """
    try:
      from dateutil.parser import parse
      return parse(string)
    except ImportError:
      return string
    except ValueError:
      raise ApiException(
          status=0,
          reason=(
              'Failed to parse `{0}` into a datetime object'.format(string)))

  def __deserialize_model(self, data, klass):
    """
        Deserializes list or dict to model.

        :param data: dict, list.
        :param klass: class literal.
        :return: model object.
        """

    if not klass.swagger_types and not hasattr(klass, 'get_real_child_model'):
      return data

    kwargs = {}
    if klass.swagger_types is not None:
      for attr, attr_type in iteritems(klass.swagger_types):
        if data is not None \
           and klass.attribute_map[attr] in data \
           and isinstance(data, (list, dict)):
          value = data[klass.attribute_map[attr]]
          kwargs[attr] = self.__deserialize(value, attr_type)

    instance = klass(**kwargs)

    if hasattr(instance, 'get_real_child_model'):
      klass_name = instance.get_real_child_model(data)
      if klass_name:
        instance = self.__deserialize(data, klass_name)
    return instance



# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from asyncio import AbstractEventLoop, new_event_loop, run_coroutine_threadsafe
from concurrent.futures import Future
from threading import Thread, Lock
from typing import ContextManager, Generic, TypeVar, Optional, Callable

_T = TypeVar("_T")


class _Lazy(Generic[_T]):
    _Factory = Callable[[], _T]

    _lock: Lock
    _factory: _Factory
    _impl: Optional[_T]

    def __init__(self, factory: _Factory):
        self._lock = Lock()
        self._factory = factory
        self._impl = None

    def get(self) -> _T:
        with self._lock:
            if self._impl is None:
                self._impl = self._factory()
            return self._impl


class _ManagedEventLoopImpl(ContextManager):
    _loop: AbstractEventLoop
    _thread: Thread

    def __init__(self, name=None):
        self._loop = new_event_loop()
        self._thread = Thread(
            target=lambda: self._loop.run_forever(), name=name, daemon=True
        )

    def __enter__(self):
        self._thread.start()
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self._loop.call_soon_threadsafe(self._loop.stop)
        self._thread.join()

    def submit(self, coro) -> Future:
        return run_coroutine_threadsafe(coro, self._loop)


# TODO(user): Remove when underlying issue is fixed.
# This is a workaround for https://github.com/grpc/grpc/issues/25364, a grpc
# issue which prevents grpc-asyncio working with multiple event loops in the
# same process. This workaround enables multiple topic publishing as well as
# publish/subscribe from the same process, but does not enable use with other
# grpc-asyncio clients. Once this issue is fixed, roll back the PR which
# introduced this to return to a single event loop per client for isolation.
_global_event_loop: _Lazy[_ManagedEventLoopImpl] = _Lazy(
    lambda: _ManagedEventLoopImpl(name="PubSubLiteEventLoopThread").__enter__()
)


class ManagedEventLoop(ContextManager):
    _loop: _ManagedEventLoopImpl

    def __init__(self, name=None):
        self._loop = _global_event_loop.get()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        pass

    def submit(self, coro) -> Future:
        return self._loop.submit(coro)



# -*- coding: utf-8 -*- #
# Copyright 2019 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for creating security policies rules."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute import org_security_policy_rule_utils as rule_utils
from googlecloudsdk.api_lib.compute.org_security_policies import client
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute.org_security_policies import flags
from googlecloudsdk.command_lib.compute.org_security_policies import org_security_policies_utils
import six


@base.ReleaseTracks(base.ReleaseTrack.BETA)
class Create(base.CreateCommand):
  r"""Create a Compute Engine security policy rule.

  *{command}* is used to create organization security policy rules.
  """

  ORG_SECURITY_POLICY_ARG = None

  @classmethod
  def Args(cls, parser):
    cls.ORG_SECURITY_POLICY_ARG = flags.OrgSecurityPolicyRuleArgument(
        required=True, operation='create')
    cls.ORG_SECURITY_POLICY_ARG.AddArgument(parser, operation_type='create')
    flags.AddAction(parser)
    flags.AddSecurityPolicyId(parser, operation='inserted')
    flags.AddSrcIpRanges(parser)
    flags.AddDestIpRanges(parser)
    flags.AddLayer4Configs(parser)
    flags.AddDirection(parser)
    flags.AddEnableLogging(parser)
    flags.AddTargetResources(parser)
    flags.AddTargetServiceAccounts(parser)
    flags.AddDescription(parser)
    flags.AddOrganization(parser, required=False)
    parser.display_info.AddCacheUpdater(flags.OrgSecurityPoliciesCompleter)

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    ref = self.ORG_SECURITY_POLICY_ARG.ResolveAsResource(
        args, holder.resources, with_project=False)
    security_policy_rule_client = client.OrgSecurityPolicyRule(
        ref=ref,
        compute_client=holder.client,
        resources=holder.resources,
        version=six.text_type(self.ReleaseTrack()).lower())
    src_ip_ranges = []
    dest_ip_ranges = []
    dest_ports = []
    layer4_configs = []
    target_resources = []
    target_service_accounts = []
    enable_logging = False
    if args.IsSpecified('src_ip_ranges'):
      src_ip_ranges = args.src_ip_ranges
    if args.IsSpecified('dest_ip_ranges'):
      dest_ip_ranges = args.dest_ip_ranges
    if self.ReleaseTrack() == base.ReleaseTrack.ALPHA and args.IsSpecified(
        'dest_ports'):
      dest_ports = args.dest_ports
    if args.IsSpecified('layer4_configs'):
      layer4_configs = args.layer4_configs
    if args.IsSpecified('target_resources'):
      target_resources = args.target_resources
    if args.IsSpecified('target_service_accounts'):
      target_service_accounts = args.target_service_accounts
    if args.IsSpecified('enable_logging'):
      enable_logging = True

    dest_ports_list = rule_utils.ParseDestPorts(dest_ports,
                                                holder.client.messages)
    layer4_config_list = rule_utils.ParseLayer4Configs(layer4_configs,
                                                       holder.client.messages)
    if self.ReleaseTrack() == base.ReleaseTrack.ALPHA:
      matcher = holder.client.messages.SecurityPolicyRuleMatcher(
          versionedExpr=holder.client.messages.SecurityPolicyRuleMatcher
          .VersionedExprValueValuesEnum.FIREWALL,
          config=holder.client.messages.SecurityPolicyRuleMatcherConfig(
              srcIpRanges=src_ip_ranges,
              destIpRanges=dest_ip_ranges,
              destPorts=dest_ports_list,
              layer4Configs=layer4_config_list))
    else:
      matcher = holder.client.messages.SecurityPolicyRuleMatcher(
          versionedExpr=holder.client.messages.SecurityPolicyRuleMatcher
          .VersionedExprValueValuesEnum.FIREWALL,
          config=holder.client.messages.SecurityPolicyRuleMatcherConfig(
              srcIpRanges=src_ip_ranges,
              destIpRanges=dest_ip_ranges,
              layer4Configs=layer4_config_list))
    traffic_direct = holder.client.messages.SecurityPolicyRule.DirectionValueValuesEnum.INGRESS
    if args.IsSpecified('direction'):
      if args.direction == 'INGRESS':
        traffic_direct = holder.client.messages.SecurityPolicyRule.DirectionValueValuesEnum.INGRESS
      else:
        traffic_direct = holder.client.messages.SecurityPolicyRule.DirectionValueValuesEnum.EGRESS

    security_policy_rule = holder.client.messages.SecurityPolicyRule(
        priority=rule_utils.ConvertPriorityToInt(ref.Name()),
        action=args.action,
        match=matcher,
        direction=traffic_direct,
        targetResources=target_resources,
        targetServiceAccounts=target_service_accounts,
        description=args.description,
        enableLogging=enable_logging)

    security_policy_id = org_security_policies_utils.GetSecurityPolicyId(
        security_policy_rule_client,
        args.security_policy,
        organization=args.organization)
    return security_policy_rule_client.Create(
        security_policy=security_policy_id,
        security_policy_rule=security_policy_rule)


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class CreateAlpha(Create):
  r"""Create a Compute Engine security policy rule.

  *{command}* is used to create organization security policy rules.
  """

  ORG_SECURITY_POLICY_ARG = None

  @classmethod
  def Args(cls, parser):
    cls.ORG_SECURITY_POLICY_ARG = flags.OrgSecurityPolicyRuleArgument(
        required=True, operation='create')
    cls.ORG_SECURITY_POLICY_ARG.AddArgument(parser, operation_type='create')
    flags.AddAction(parser)
    flags.AddSecurityPolicyId(parser, operation='inserted')
    flags.AddSrcIpRanges(parser)
    flags.AddDestIpRanges(parser)
    flags.AddDestPorts(parser)
    flags.AddLayer4Configs(parser)
    flags.AddDirection(parser)
    flags.AddEnableLogging(parser)
    flags.AddTargetResources(parser)
    flags.AddTargetServiceAccounts(parser)
    flags.AddDescription(parser)
    flags.AddOrganization(parser, required=False)
    parser.display_info.AddCacheUpdater(flags.OrgSecurityPoliciesCompleter)


Create.detailed_help = {
    'EXAMPLES':
        """\
    To create a rule with priority ``10" in an organization security policy with ID
    ``123456789", run:

      $ {command} create 10 --security-policy=123456789 --action=allow
      --description=example-rule
    """,
}



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for updating organization firewall policies."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute.firewall_policies import client
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute.firewall_policies import firewall_policies_utils
from googlecloudsdk.command_lib.compute.firewall_policies import flags
import six


class Update(base.UpdateCommand):
  """Update a Compute Engine organization firewall policy.

  *{command}* is used to update organization firewall policies. An organization
  firewall policy is a set of rules that controls access to various resources.
  """

  FIREWALL_POLICY_ARG = None

  @classmethod
  def Args(cls, parser):
    cls.FIREWALL_POLICY_ARG = flags.FirewallPolicyArgument(
        required=True, operation='update')
    cls.FIREWALL_POLICY_ARG.AddArgument(parser, operation_type='update')
    flags.AddArgsUpdateFirewallPolicy(parser)

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    ref = self.FIREWALL_POLICY_ARG.ResolveAsResource(
        args, holder.resources, with_project=False)
    org_firewall_policy = client.OrgFirewallPolicy(
        ref=ref,
        compute_client=holder.client,
        resources=holder.resources,
        version=six.text_type(self.ReleaseTrack()).lower())
    fp_id = firewall_policies_utils.GetFirewallPolicyId(
        org_firewall_policy, ref.Name(), organization=args.organization)
    existing_firewall_policy = org_firewall_policy.Describe(
        fp_id=fp_id, only_generate_request=False)[0]
    firewall_policy = holder.client.messages.FirewallPolicy(
        description=args.description,
        fingerprint=existing_firewall_policy.fingerprint)

    return org_firewall_policy.Update(
        fp_id=fp_id,
        only_generate_request=False,
        firewall_policy=firewall_policy)


Update.detailed_help = {
    'EXAMPLES':
        """\
    To update an organization firewall policy with ID ``123456789" to change the
    description to ``New description", run:

      $ {command} 123456789 --description='New description'
    """,
}



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The command to update Service Mesh Feature."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope import base as gbase
from googlecloudsdk.calliope import exceptions as calliope_exceptions
from googlecloudsdk.command_lib.container.fleet.features import base
from googlecloudsdk.core import exceptions
from googlecloudsdk.core.console import console_io


@gbase.ReleaseTracks(gbase.ReleaseTrack.ALPHA)
class Update(base.UpdateCommand):
  """Update the configuration of the Service Mesh Feature.

  Update the Service Mesh Feature Spec of a Membership.

  ## EXAMPLES

  To update the control plane management of a Membership named MEMBERSHIP, run:

    $ {command} --membership=MEMBERSHIP
      --control-plane=CONTROL_PLANE
  """

  feature_name = 'servicemesh'

  @staticmethod
  def Args(parser):
    parser.add_argument(
        '--membership',
        type=str,
        help='The Membership name to update.',
    )
    parser.add_argument(
        '--control-plane',
        choices=['automatic', 'manual'],
        help='The control plane management to update to.',
        required=True
    )

  def Run(self, args):
    memberships = base.ListMemberships()
    if not memberships:
      raise exceptions.Error('No Memberships available in the fleet.')
    membership = args.membership
    if not membership:
      if console_io.CanPrompt():
        index = console_io.PromptChoice(
            options=memberships,
            message='Please specify a Membership:\n',
            cancel_option=True)
        membership = memberships[index]
      else:
        raise calliope_exceptions.RequiredArgumentException(
            '--membership',
            ('Cannot prompt a console for membership. Membership is required. '
             'Please specify `--membership` to select a membership.'))
    else:
      if membership not in memberships:
        raise exceptions.Error(
            'Membership {} does not exist in the fleet.'.format(membership))
    membership = self.MembershipResourceName(membership)

    f = self.GetFeature()
    patch = self.messages.MembershipFeatureSpec()

    for name, spec in self.hubclient.ToPyDict(f.membershipSpecs).items():
      if name == membership and spec:
        patch = spec
    if not patch.mesh:
      patch.mesh = self.messages.ServiceMeshMembershipSpec()

    control_plane = (
        self.messages.ServiceMeshMembershipSpec.ControlPlaneValueValuesEnum(
            'MANUAL'))
    if args.control_plane == 'automatic':
      control_plane = (
          self.messages.ServiceMeshMembershipSpec.ControlPlaneValueValuesEnum(
              'AUTOMATIC'))
    patch.mesh.controlPlane = control_plane

    f = self.messages.Feature(
        membershipSpecs=self.hubclient.ToMembershipSpecs({membership: patch}))
    self.Update(['membershipSpecs'], f)



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to list available Kuberun Development Kits."""
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import json

from googlecloudsdk.api_lib.kuberun import devkit
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.kuberun import kuberun_command

_DETAILED_HELP = {
    'EXAMPLES':
        """
        To show all available Development Kits, run:

            $ {command}
        """,
}


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class List(kuberun_command.KubeRunCommand, base.ListCommand):
  """Lists available Development Kits."""

  detailed_help = _DETAILED_HELP
  flags = []

  @classmethod
  def Args(cls, parser):
    super(List, cls).Args(parser)
    base.ListCommand._Flags(parser)
    base.URI_FLAG.RemoveFromParser(parser)
    columns = ['id', 'name', 'description', 'version']
    parser.display_info.AddFormat('table({})'.format(','.join(columns)))

  def Command(self):
    return ['devkits', 'list']

  def SuccessResult(self, out, args):
    if out:
      json_list = json.loads(out)
      return [devkit.DevKit.FromJSON(x) for x in json_list]
    else:
      return []



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Disable-inherit command for the Org Policy CLI."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import copy

from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.org_policies import exceptions
from googlecloudsdk.command_lib.org_policies import interfaces


@base.Hidden
@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class DisableInherit(interfaces.OrgPolicyGetAndUpdateCommand):
  r"""Disable inheritance of policy behavior from parent resources.

  Disables inheritance of policy behavior from parent resources.

  ## EXAMPLES

  To disable inheritance of policy behavior on the policy associated with the
  constraint 'gcp.resourceLocations' and the Project 'foo-project', run:

    $ {command} gcp.resourceLocations --project=foo-project
  """

  def UpdatePolicy(self, policy, args):
    """Sets the inheritFromParent field on the policy to False.

    Args:
      policy: messages.GoogleCloudOrgpolicyV2alpha1Policy, The policy to be
        updated.
      args: argparse.Namespace, An object that contains the values for the
        arguments specified in the Args method.

    Returns:
      The updated policy.
    """
    if not policy.spec.rules:
      raise exceptions.OperationNotSupportedError(
          'Cannot disable inherit on an empty policy. To create a policy use allow/deny, enable/disable_enforce or set_policy.'
      )

    new_policy = copy.deepcopy(policy)
    new_policy.spec.inheritFromParent = False

    return new_policy



# coding: utf-8
"""
    Kubernetes

    No description provided (generated by Swagger Codegen
    https://github.com/swagger-api/swagger-codegen)

    OpenAPI spec version: v1.14.4

    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

# import models into model package
from .admissionregistration_v1beta1_service_reference import AdmissionregistrationV1beta1ServiceReference
from .admissionregistration_v1beta1_webhook_client_config import AdmissionregistrationV1beta1WebhookClientConfig
from .apiextensions_v1beta1_service_reference import ApiextensionsV1beta1ServiceReference
from .apiextensions_v1beta1_webhook_client_config import ApiextensionsV1beta1WebhookClientConfig
from .apiregistration_v1beta1_service_reference import ApiregistrationV1beta1ServiceReference
from .apps_v1beta1_deployment import AppsV1beta1Deployment
from .apps_v1beta1_deployment_condition import AppsV1beta1DeploymentCondition
from .apps_v1beta1_deployment_list import AppsV1beta1DeploymentList
from .apps_v1beta1_deployment_rollback import AppsV1beta1DeploymentRollback
from .apps_v1beta1_deployment_spec import AppsV1beta1DeploymentSpec
from .apps_v1beta1_deployment_status import AppsV1beta1DeploymentStatus
from .apps_v1beta1_deployment_strategy import AppsV1beta1DeploymentStrategy
from .apps_v1beta1_rollback_config import AppsV1beta1RollbackConfig
from .apps_v1beta1_rolling_update_deployment import AppsV1beta1RollingUpdateDeployment
from .apps_v1beta1_scale import AppsV1beta1Scale
from .apps_v1beta1_scale_spec import AppsV1beta1ScaleSpec
from .apps_v1beta1_scale_status import AppsV1beta1ScaleStatus
from .extensions_v1beta1_allowed_csi_driver import ExtensionsV1beta1AllowedCSIDriver
from .extensions_v1beta1_allowed_flex_volume import ExtensionsV1beta1AllowedFlexVolume
from .extensions_v1beta1_allowed_host_path import ExtensionsV1beta1AllowedHostPath
from .extensions_v1beta1_deployment import ExtensionsV1beta1Deployment
from .extensions_v1beta1_deployment_condition import ExtensionsV1beta1DeploymentCondition
from .extensions_v1beta1_deployment_list import ExtensionsV1beta1DeploymentList
from .extensions_v1beta1_deployment_rollback import ExtensionsV1beta1DeploymentRollback
from .extensions_v1beta1_deployment_spec import ExtensionsV1beta1DeploymentSpec
from .extensions_v1beta1_deployment_status import ExtensionsV1beta1DeploymentStatus
from .extensions_v1beta1_deployment_strategy import ExtensionsV1beta1DeploymentStrategy
from .extensions_v1beta1_fs_group_strategy_options import ExtensionsV1beta1FSGroupStrategyOptions
from .extensions_v1beta1_http_ingress_path import ExtensionsV1beta1HTTPIngressPath
from .extensions_v1beta1_http_ingress_rule_value import ExtensionsV1beta1HTTPIngressRuleValue
from .extensions_v1beta1_host_port_range import ExtensionsV1beta1HostPortRange
from .extensions_v1beta1_id_range import ExtensionsV1beta1IDRange
from .extensions_v1beta1_ingress import ExtensionsV1beta1Ingress
from .extensions_v1beta1_ingress_backend import ExtensionsV1beta1IngressBackend
from .extensions_v1beta1_ingress_list import ExtensionsV1beta1IngressList
from .extensions_v1beta1_ingress_rule import ExtensionsV1beta1IngressRule
from .extensions_v1beta1_ingress_spec import ExtensionsV1beta1IngressSpec
from .extensions_v1beta1_ingress_status import ExtensionsV1beta1IngressStatus
from .extensions_v1beta1_ingress_tls import ExtensionsV1beta1IngressTLS
from .extensions_v1beta1_pod_security_policy import ExtensionsV1beta1PodSecurityPolicy
from .extensions_v1beta1_pod_security_policy_list import ExtensionsV1beta1PodSecurityPolicyList
from .extensions_v1beta1_pod_security_policy_spec import ExtensionsV1beta1PodSecurityPolicySpec
from .extensions_v1beta1_rollback_config import ExtensionsV1beta1RollbackConfig
from .extensions_v1beta1_rolling_update_deployment import ExtensionsV1beta1RollingUpdateDeployment
from .extensions_v1beta1_run_as_group_strategy_options import ExtensionsV1beta1RunAsGroupStrategyOptions
from .extensions_v1beta1_run_as_user_strategy_options import ExtensionsV1beta1RunAsUserStrategyOptions
from .extensions_v1beta1_se_linux_strategy_options import ExtensionsV1beta1SELinuxStrategyOptions
from .extensions_v1beta1_scale import ExtensionsV1beta1Scale
from .extensions_v1beta1_scale_spec import ExtensionsV1beta1ScaleSpec
from .extensions_v1beta1_scale_status import ExtensionsV1beta1ScaleStatus
from .extensions_v1beta1_supplemental_groups_strategy_options import ExtensionsV1beta1SupplementalGroupsStrategyOptions
from .networking_v1beta1_http_ingress_path import NetworkingV1beta1HTTPIngressPath
from .networking_v1beta1_http_ingress_rule_value import NetworkingV1beta1HTTPIngressRuleValue
from .networking_v1beta1_ingress import NetworkingV1beta1Ingress
from .networking_v1beta1_ingress_backend import NetworkingV1beta1IngressBackend
from .networking_v1beta1_ingress_list import NetworkingV1beta1IngressList
from .networking_v1beta1_ingress_rule import NetworkingV1beta1IngressRule
from .networking_v1beta1_ingress_spec import NetworkingV1beta1IngressSpec
from .networking_v1beta1_ingress_status import NetworkingV1beta1IngressStatus
from .networking_v1beta1_ingress_tls import NetworkingV1beta1IngressTLS
from .policy_v1beta1_allowed_csi_driver import PolicyV1beta1AllowedCSIDriver
from .policy_v1beta1_allowed_flex_volume import PolicyV1beta1AllowedFlexVolume
from .policy_v1beta1_allowed_host_path import PolicyV1beta1AllowedHostPath
from .policy_v1beta1_fs_group_strategy_options import PolicyV1beta1FSGroupStrategyOptions
from .policy_v1beta1_host_port_range import PolicyV1beta1HostPortRange
from .policy_v1beta1_id_range import PolicyV1beta1IDRange
from .policy_v1beta1_pod_security_policy import PolicyV1beta1PodSecurityPolicy
from .policy_v1beta1_pod_security_policy_list import PolicyV1beta1PodSecurityPolicyList
from .policy_v1beta1_pod_security_policy_spec import PolicyV1beta1PodSecurityPolicySpec
from .policy_v1beta1_run_as_group_strategy_options import PolicyV1beta1RunAsGroupStrategyOptions
from .policy_v1beta1_run_as_user_strategy_options import PolicyV1beta1RunAsUserStrategyOptions
from .policy_v1beta1_se_linux_strategy_options import PolicyV1beta1SELinuxStrategyOptions
from .policy_v1beta1_supplemental_groups_strategy_options import PolicyV1beta1SupplementalGroupsStrategyOptions
from .runtime_raw_extension import RuntimeRawExtension
from .v1_api_group import V1APIGroup
from .v1_api_group_list import V1APIGroupList
from .v1_api_resource import V1APIResource
from .v1_api_resource_list import V1APIResourceList
from .v1_api_service import V1APIService
from .v1_api_service_condition import V1APIServiceCondition
from .v1_api_service_list import V1APIServiceList
from .v1_api_service_spec import V1APIServiceSpec
from .v1_api_service_status import V1APIServiceStatus
from .v1_api_versions import V1APIVersions
from .v1_aws_elastic_block_store_volume_source import V1AWSElasticBlockStoreVolumeSource
from .v1_affinity import V1Affinity
from .v1_aggregation_rule import V1AggregationRule
from .v1_attached_volume import V1AttachedVolume
from .v1_azure_disk_volume_source import V1AzureDiskVolumeSource
from .v1_azure_file_persistent_volume_source import V1AzureFilePersistentVolumeSource
from .v1_azure_file_volume_source import V1AzureFileVolumeSource
from .v1_binding import V1Binding
from .v1_csi_persistent_volume_source import V1CSIPersistentVolumeSource
from .v1_csi_volume_source import V1CSIVolumeSource
from .v1_capabilities import V1Capabilities
from .v1_ceph_fs_persistent_volume_source import V1CephFSPersistentVolumeSource
from .v1_ceph_fs_volume_source import V1CephFSVolumeSource
from .v1_cinder_persistent_volume_source import V1CinderPersistentVolumeSource
from .v1_cinder_volume_source import V1CinderVolumeSource
from .v1_client_ip_config import V1ClientIPConfig
from .v1_cluster_role import V1ClusterRole
from .v1_cluster_role_binding import V1ClusterRoleBinding
from .v1_cluster_role_binding_list import V1ClusterRoleBindingList
from .v1_cluster_role_list import V1ClusterRoleList
from .v1_component_condition import V1ComponentCondition
from .v1_component_status import V1ComponentStatus
from .v1_component_status_list import V1ComponentStatusList
from .v1_config_map import V1ConfigMap
from .v1_config_map_env_source import V1ConfigMapEnvSource
from .v1_config_map_key_selector import V1ConfigMapKeySelector
from .v1_config_map_list import V1ConfigMapList
from .v1_config_map_node_config_source import V1ConfigMapNodeConfigSource
from .v1_config_map_projection import V1ConfigMapProjection
from .v1_config_map_volume_source import V1ConfigMapVolumeSource
from .v1_container import V1Container
from .v1_container_image import V1ContainerImage
from .v1_container_port import V1ContainerPort
from .v1_container_state import V1ContainerState
from .v1_container_state_running import V1ContainerStateRunning
from .v1_container_state_terminated import V1ContainerStateTerminated
from .v1_container_state_waiting import V1ContainerStateWaiting
from .v1_container_status import V1ContainerStatus
from .v1_controller_revision import V1ControllerRevision
from .v1_controller_revision_list import V1ControllerRevisionList
from .v1_cross_version_object_reference import V1CrossVersionObjectReference
from .v1_daemon_endpoint import V1DaemonEndpoint
from .v1_daemon_set import V1DaemonSet
from .v1_daemon_set_condition import V1DaemonSetCondition
from .v1_daemon_set_list import V1DaemonSetList
from .v1_daemon_set_spec import V1DaemonSetSpec
from .v1_daemon_set_status import V1DaemonSetStatus
from .v1_daemon_set_update_strategy import V1DaemonSetUpdateStrategy
from .v1_delete_options import V1DeleteOptions
from .v1_deployment import V1Deployment
from .v1_deployment_condition import V1DeploymentCondition
from .v1_deployment_list import V1DeploymentList
from .v1_deployment_spec import V1DeploymentSpec
from .v1_deployment_status import V1DeploymentStatus
from .v1_deployment_strategy import V1DeploymentStrategy
from .v1_downward_api_projection import V1DownwardAPIProjection
from .v1_downward_api_volume_file import V1DownwardAPIVolumeFile
from .v1_downward_api_volume_source import V1DownwardAPIVolumeSource
from .v1_empty_dir_volume_source import V1EmptyDirVolumeSource
from .v1_endpoint_address import V1EndpointAddress
from .v1_endpoint_port import V1EndpointPort
from .v1_endpoint_subset import V1EndpointSubset
from .v1_endpoints import V1Endpoints
from .v1_endpoints_list import V1EndpointsList
from .v1_env_from_source import V1EnvFromSource
from .v1_env_var import V1EnvVar
from .v1_env_var_source import V1EnvVarSource
from .v1_event import V1Event
from .v1_event_list import V1EventList
from .v1_event_series import V1EventSeries
from .v1_event_source import V1EventSource
from .v1_exec_action import V1ExecAction
from .v1_fc_volume_source import V1FCVolumeSource
from .v1_flex_persistent_volume_source import V1FlexPersistentVolumeSource
from .v1_flex_volume_source import V1FlexVolumeSource
from .v1_flocker_volume_source import V1FlockerVolumeSource
from .v1_gce_persistent_disk_volume_source import V1GCEPersistentDiskVolumeSource
from .v1_git_repo_volume_source import V1GitRepoVolumeSource
from .v1_glusterfs_persistent_volume_source import V1GlusterfsPersistentVolumeSource
from .v1_glusterfs_volume_source import V1GlusterfsVolumeSource
from .v1_group_version_for_discovery import V1GroupVersionForDiscovery
from .v1_http_get_action import V1HTTPGetAction
from .v1_http_header import V1HTTPHeader
from .v1_handler import V1Handler
from .v1_horizontal_pod_autoscaler import V1HorizontalPodAutoscaler
from .v1_horizontal_pod_autoscaler_list import V1HorizontalPodAutoscalerList
from .v1_horizontal_pod_autoscaler_spec import V1HorizontalPodAutoscalerSpec
from .v1_horizontal_pod_autoscaler_status import V1HorizontalPodAutoscalerStatus
from .v1_host_alias import V1HostAlias
from .v1_host_path_volume_source import V1HostPathVolumeSource
from .v1_ip_block import V1IPBlock
from .v1_iscsi_persistent_volume_source import V1ISCSIPersistentVolumeSource
from .v1_iscsi_volume_source import V1ISCSIVolumeSource
from .v1_initializer import V1Initializer
from .v1_initializers import V1Initializers
from .v1_job import V1Job
from .v1_job_condition import V1JobCondition
from .v1_job_list import V1JobList
from .v1_job_spec import V1JobSpec
from .v1_job_status import V1JobStatus
from .v1_key_to_path import V1KeyToPath
from .v1_label_selector import V1LabelSelector
from .v1_label_selector_requirement import V1LabelSelectorRequirement
from .v1_lease import V1Lease
from .v1_lease_list import V1LeaseList
from .v1_lease_spec import V1LeaseSpec
from .v1_lifecycle import V1Lifecycle
from .v1_limit_range import V1LimitRange
from .v1_limit_range_item import V1LimitRangeItem
from .v1_limit_range_list import V1LimitRangeList
from .v1_limit_range_spec import V1LimitRangeSpec
from .v1_list_meta import V1ListMeta
from .v1_load_balancer_ingress import V1LoadBalancerIngress
from .v1_load_balancer_status import V1LoadBalancerStatus
from .v1_local_object_reference import V1LocalObjectReference
from .v1_local_subject_access_review import V1LocalSubjectAccessReview
from .v1_local_volume_source import V1LocalVolumeSource
from .v1_managed_fields_entry import V1ManagedFieldsEntry
from .v1_nfs_volume_source import V1NFSVolumeSource
from .v1_namespace import V1Namespace
from .v1_namespace_list import V1NamespaceList
from .v1_namespace_spec import V1NamespaceSpec
from .v1_namespace_status import V1NamespaceStatus
from .v1_network_policy import V1NetworkPolicy
from .v1_network_policy_egress_rule import V1NetworkPolicyEgressRule
from .v1_network_policy_ingress_rule import V1NetworkPolicyIngressRule
from .v1_network_policy_list import V1NetworkPolicyList
from .v1_network_policy_peer import V1NetworkPolicyPeer
from .v1_network_policy_port import V1NetworkPolicyPort
from .v1_network_policy_spec import V1NetworkPolicySpec
from .v1_node import V1Node
from .v1_node_address import V1NodeAddress
from .v1_node_affinity import V1NodeAffinity
from .v1_node_condition import V1NodeCondition
from .v1_node_config_source import V1NodeConfigSource
from .v1_node_config_status import V1NodeConfigStatus
from .v1_node_daemon_endpoints import V1NodeDaemonEndpoints
from .v1_node_list import V1NodeList
from .v1_node_selector import V1NodeSelector
from .v1_node_selector_requirement import V1NodeSelectorRequirement
from .v1_node_selector_term import V1NodeSelectorTerm
from .v1_node_spec import V1NodeSpec
from .v1_node_status import V1NodeStatus
from .v1_node_system_info import V1NodeSystemInfo
from .v1_non_resource_attributes import V1NonResourceAttributes
from .v1_non_resource_rule import V1NonResourceRule
from .v1_object_field_selector import V1ObjectFieldSelector
from .v1_object_meta import V1ObjectMeta
from .v1_object_reference import V1ObjectReference
from .v1_owner_reference import V1OwnerReference
from .v1_persistent_volume import V1PersistentVolume
from .v1_persistent_volume_claim import V1PersistentVolumeClaim
from .v1_persistent_volume_claim_condition import V1PersistentVolumeClaimCondition
from .v1_persistent_volume_claim_list import V1PersistentVolumeClaimList
from .v1_persistent_volume_claim_spec import V1PersistentVolumeClaimSpec
from .v1_persistent_volume_claim_status import V1PersistentVolumeClaimStatus
from .v1_persistent_volume_claim_volume_source import V1PersistentVolumeClaimVolumeSource
from .v1_persistent_volume_list import V1PersistentVolumeList
from .v1_persistent_volume_spec import V1PersistentVolumeSpec
from .v1_persistent_volume_status import V1PersistentVolumeStatus
from .v1_photon_persistent_disk_volume_source import V1PhotonPersistentDiskVolumeSource
from .v1_pod import V1Pod
from .v1_pod_affinity import V1PodAffinity
from .v1_pod_affinity_term import V1PodAffinityTerm
from .v1_pod_anti_affinity import V1PodAntiAffinity
from .v1_pod_condition import V1PodCondition
from .v1_pod_dns_config import V1PodDNSConfig
from .v1_pod_dns_config_option import V1PodDNSConfigOption
from .v1_pod_list import V1PodList
from .v1_pod_readiness_gate import V1PodReadinessGate
from .v1_pod_security_context import V1PodSecurityContext
from .v1_pod_spec import V1PodSpec
from .v1_pod_status import V1PodStatus
from .v1_pod_template import V1PodTemplate
from .v1_pod_template_list import V1PodTemplateList
from .v1_pod_template_spec import V1PodTemplateSpec
from .v1_policy_rule import V1PolicyRule
from .v1_portworx_volume_source import V1PortworxVolumeSource
from .v1_preconditions import V1Preconditions
from .v1_preferred_scheduling_term import V1PreferredSchedulingTerm
from .v1_priority_class import V1PriorityClass
from .v1_priority_class_list import V1PriorityClassList
from .v1_probe import V1Probe
from .v1_projected_volume_source import V1ProjectedVolumeSource
from .v1_quobyte_volume_source import V1QuobyteVolumeSource
from .v1_rbd_persistent_volume_source import V1RBDPersistentVolumeSource
from .v1_rbd_volume_source import V1RBDVolumeSource
from .v1_replica_set import V1ReplicaSet
from .v1_replica_set_condition import V1ReplicaSetCondition
from .v1_replica_set_list import V1ReplicaSetList
from .v1_replica_set_spec import V1ReplicaSetSpec
from .v1_replica_set_status import V1ReplicaSetStatus
from .v1_replication_controller import V1ReplicationController
from .v1_replication_controller_condition import V1ReplicationControllerCondition
from .v1_replication_controller_list import V1ReplicationControllerList
from .v1_replication_controller_spec import V1ReplicationControllerSpec
from .v1_replication_controller_status import V1ReplicationControllerStatus
from .v1_resource_attributes import V1ResourceAttributes
from .v1_resource_field_selector import V1ResourceFieldSelector
from .v1_resource_quota import V1ResourceQuota
from .v1_resource_quota_list import V1ResourceQuotaList
from .v1_resource_quota_spec import V1ResourceQuotaSpec
from .v1_resource_quota_status import V1ResourceQuotaStatus
from .v1_resource_requirements import V1ResourceRequirements
from .v1_resource_rule import V1ResourceRule
from .v1_role import V1Role
from .v1_role_binding import V1RoleBinding
from .v1_role_binding_list import V1RoleBindingList
from .v1_role_list import V1RoleList
from .v1_role_ref import V1RoleRef
from .v1_rolling_update_daemon_set import V1RollingUpdateDaemonSet
from .v1_rolling_update_deployment import V1RollingUpdateDeployment
from .v1_rolling_update_stateful_set_strategy import V1RollingUpdateStatefulSetStrategy
from .v1_se_linux_options import V1SELinuxOptions
from .v1_scale import V1Scale
from .v1_scale_io_persistent_volume_source import V1ScaleIOPersistentVolumeSource
from .v1_scale_io_volume_source import V1ScaleIOVolumeSource
from .v1_scale_spec import V1ScaleSpec
from .v1_scale_status import V1ScaleStatus
from .v1_scope_selector import V1ScopeSelector
from .v1_scoped_resource_selector_requirement import V1ScopedResourceSelectorRequirement
from .v1_secret import V1Secret
from .v1_secret_env_source import V1SecretEnvSource
from .v1_secret_key_selector import V1SecretKeySelector
from .v1_secret_list import V1SecretList
from .v1_secret_projection import V1SecretProjection
from .v1_secret_reference import V1SecretReference
from .v1_secret_volume_source import V1SecretVolumeSource
from .v1_security_context import V1SecurityContext
from .v1_self_subject_access_review import V1SelfSubjectAccessReview
from .v1_self_subject_access_review_spec import V1SelfSubjectAccessReviewSpec
from .v1_self_subject_rules_review import V1SelfSubjectRulesReview
from .v1_self_subject_rules_review_spec import V1SelfSubjectRulesReviewSpec
from .v1_server_address_by_client_cidr import V1ServerAddressByClientCIDR
from .v1_service import V1Service
from .v1_service_account import V1ServiceAccount
from .v1_service_account_list import V1ServiceAccountList
from .v1_service_account_token_projection import V1ServiceAccountTokenProjection
from .v1_service_list import V1ServiceList
from .v1_service_port import V1ServicePort
from .v1_service_reference import V1ServiceReference
from .v1_service_spec import V1ServiceSpec
from .v1_service_status import V1ServiceStatus
from .v1_session_affinity_config import V1SessionAffinityConfig
from .v1_stateful_set import V1StatefulSet
from .v1_stateful_set_condition import V1StatefulSetCondition
from .v1_stateful_set_list import V1StatefulSetList
from .v1_stateful_set_spec import V1StatefulSetSpec
from .v1_stateful_set_status import V1StatefulSetStatus
from .v1_stateful_set_update_strategy import V1StatefulSetUpdateStrategy
from .v1_status import V1Status
from .v1_status_cause import V1StatusCause
from .v1_status_details import V1StatusDetails
from .v1_storage_class import V1StorageClass
from .v1_storage_class_list import V1StorageClassList
from .v1_storage_os_persistent_volume_source import V1StorageOSPersistentVolumeSource
from .v1_storage_os_volume_source import V1StorageOSVolumeSource
from .v1_subject import V1Subject
from .v1_subject_access_review import V1SubjectAccessReview
from .v1_subject_access_review_spec import V1SubjectAccessReviewSpec
from .v1_subject_access_review_status import V1SubjectAccessReviewStatus
from .v1_subject_rules_review_status import V1SubjectRulesReviewStatus
from .v1_sysctl import V1Sysctl
from .v1_tcp_socket_action import V1TCPSocketAction
from .v1_taint import V1Taint
from .v1_token_review import V1TokenReview
from .v1_token_review_spec import V1TokenReviewSpec
from .v1_token_review_status import V1TokenReviewStatus
from .v1_toleration import V1Toleration
from .v1_topology_selector_label_requirement import V1TopologySelectorLabelRequirement
from .v1_topology_selector_term import V1TopologySelectorTerm
from .v1_typed_local_object_reference import V1TypedLocalObjectReference
from .v1_user_info import V1UserInfo
from .v1_volume import V1Volume
from .v1_volume_attachment import V1VolumeAttachment
from .v1_volume_attachment_list import V1VolumeAttachmentList
from .v1_volume_attachment_source import V1VolumeAttachmentSource
from .v1_volume_attachment_spec import V1VolumeAttachmentSpec
from .v1_volume_attachment_status import V1VolumeAttachmentStatus
from .v1_volume_device import V1VolumeDevice
from .v1_volume_error import V1VolumeError
from .v1_volume_mount import V1VolumeMount
from .v1_volume_node_affinity import V1VolumeNodeAffinity
from .v1_volume_projection import V1VolumeProjection
from .v1_vsphere_virtual_disk_volume_source import V1VsphereVirtualDiskVolumeSource
from .v1_watch_event import V1WatchEvent
from .v1_weighted_pod_affinity_term import V1WeightedPodAffinityTerm
from .v1alpha1_aggregation_rule import V1alpha1AggregationRule
from .v1alpha1_audit_sink import V1alpha1AuditSink
from .v1alpha1_audit_sink_list import V1alpha1AuditSinkList
from .v1alpha1_audit_sink_spec import V1alpha1AuditSinkSpec
from .v1alpha1_cluster_role import V1alpha1ClusterRole
from .v1alpha1_cluster_role_binding import V1alpha1ClusterRoleBinding
from .v1alpha1_cluster_role_binding_list import V1alpha1ClusterRoleBindingList
from .v1alpha1_cluster_role_list import V1alpha1ClusterRoleList
from .v1alpha1_pod_preset import V1alpha1PodPreset
from .v1alpha1_pod_preset_list import V1alpha1PodPresetList
from .v1alpha1_pod_preset_spec import V1alpha1PodPresetSpec
from .v1alpha1_policy import V1alpha1Policy
from .v1alpha1_policy_rule import V1alpha1PolicyRule
from .v1alpha1_priority_class import V1alpha1PriorityClass
from .v1alpha1_priority_class_list import V1alpha1PriorityClassList
from .v1alpha1_role import V1alpha1Role
from .v1alpha1_role_binding import V1alpha1RoleBinding
from .v1alpha1_role_binding_list import V1alpha1RoleBindingList
from .v1alpha1_role_list import V1alpha1RoleList
from .v1alpha1_role_ref import V1alpha1RoleRef
from .v1alpha1_runtime_class import V1alpha1RuntimeClass
from .v1alpha1_runtime_class_list import V1alpha1RuntimeClassList
from .v1alpha1_runtime_class_spec import V1alpha1RuntimeClassSpec
from .v1alpha1_service_reference import V1alpha1ServiceReference
from .v1alpha1_subject import V1alpha1Subject
from .v1alpha1_volume_attachment import V1alpha1VolumeAttachment
from .v1alpha1_volume_attachment_list import V1alpha1VolumeAttachmentList
from .v1alpha1_volume_attachment_source import V1alpha1VolumeAttachmentSource
from .v1alpha1_volume_attachment_spec import V1alpha1VolumeAttachmentSpec
from .v1alpha1_volume_attachment_status import V1alpha1VolumeAttachmentStatus
from .v1alpha1_volume_error import V1alpha1VolumeError
from .v1alpha1_webhook import V1alpha1Webhook
from .v1alpha1_webhook_client_config import V1alpha1WebhookClientConfig
from .v1alpha1_webhook_throttle_config import V1alpha1WebhookThrottleConfig
from .v1beta1_api_service import V1beta1APIService
from .v1beta1_api_service_condition import V1beta1APIServiceCondition
from .v1beta1_api_service_list import V1beta1APIServiceList
from .v1beta1_api_service_spec import V1beta1APIServiceSpec
from .v1beta1_api_service_status import V1beta1APIServiceStatus
from .v1beta1_aggregation_rule import V1beta1AggregationRule
from .v1beta1_csi_driver import V1beta1CSIDriver
from .v1beta1_csi_driver_list import V1beta1CSIDriverList
from .v1beta1_csi_driver_spec import V1beta1CSIDriverSpec
from .v1beta1_csi_node import V1beta1CSINode
from .v1beta1_csi_node_driver import V1beta1CSINodeDriver
from .v1beta1_csi_node_list import V1beta1CSINodeList
from .v1beta1_csi_node_spec import V1beta1CSINodeSpec
from .v1beta1_certificate_signing_request import V1beta1CertificateSigningRequest
from .v1beta1_certificate_signing_request_condition import V1beta1CertificateSigningRequestCondition
from .v1beta1_certificate_signing_request_list import V1beta1CertificateSigningRequestList
from .v1beta1_certificate_signing_request_spec import V1beta1CertificateSigningRequestSpec
from .v1beta1_certificate_signing_request_status import V1beta1CertificateSigningRequestStatus
from .v1beta1_cluster_role import V1beta1ClusterRole
from .v1beta1_cluster_role_binding import V1beta1ClusterRoleBinding
from .v1beta1_cluster_role_binding_list import V1beta1ClusterRoleBindingList
from .v1beta1_cluster_role_list import V1beta1ClusterRoleList
from .v1beta1_controller_revision import V1beta1ControllerRevision
from .v1beta1_controller_revision_list import V1beta1ControllerRevisionList
from .v1beta1_cron_job import V1beta1CronJob
from .v1beta1_cron_job_list import V1beta1CronJobList
from .v1beta1_cron_job_spec import V1beta1CronJobSpec
from .v1beta1_cron_job_status import V1beta1CronJobStatus
from .v1beta1_custom_resource_column_definition import V1beta1CustomResourceColumnDefinition
from .v1beta1_custom_resource_conversion import V1beta1CustomResourceConversion
from .v1beta1_custom_resource_definition import V1beta1CustomResourceDefinition
from .v1beta1_custom_resource_definition_condition import V1beta1CustomResourceDefinitionCondition
from .v1beta1_custom_resource_definition_list import V1beta1CustomResourceDefinitionList
from .v1beta1_custom_resource_definition_names import V1beta1CustomResourceDefinitionNames
from .v1beta1_custom_resource_definition_spec import V1beta1CustomResourceDefinitionSpec
from .v1beta1_custom_resource_definition_status import V1beta1CustomResourceDefinitionStatus
from .v1beta1_custom_resource_definition_version import V1beta1CustomResourceDefinitionVersion
from .v1beta1_custom_resource_subresource_scale import V1beta1CustomResourceSubresourceScale
from .v1beta1_custom_resource_subresources import V1beta1CustomResourceSubresources
from .v1beta1_custom_resource_validation import V1beta1CustomResourceValidation
from .v1beta1_daemon_set import V1beta1DaemonSet
from .v1beta1_daemon_set_condition import V1beta1DaemonSetCondition
from .v1beta1_daemon_set_list import V1beta1DaemonSetList
from .v1beta1_daemon_set_spec import V1beta1DaemonSetSpec
from .v1beta1_daemon_set_status import V1beta1DaemonSetStatus
from .v1beta1_daemon_set_update_strategy import V1beta1DaemonSetUpdateStrategy
from .v1beta1_event import V1beta1Event
from .v1beta1_event_list import V1beta1EventList
from .v1beta1_event_series import V1beta1EventSeries
from .v1beta1_eviction import V1beta1Eviction
from .v1beta1_external_documentation import V1beta1ExternalDocumentation
from .v1beta1_ip_block import V1beta1IPBlock
from .v1beta1_json_schema_props import V1beta1JSONSchemaProps
from .v1beta1_job_template_spec import V1beta1JobTemplateSpec
from .v1beta1_lease import V1beta1Lease
from .v1beta1_lease_list import V1beta1LeaseList
from .v1beta1_lease_spec import V1beta1LeaseSpec
from .v1beta1_local_subject_access_review import V1beta1LocalSubjectAccessReview
from .v1beta1_mutating_webhook_configuration import V1beta1MutatingWebhookConfiguration
from .v1beta1_mutating_webhook_configuration_list import V1beta1MutatingWebhookConfigurationList
from .v1beta1_network_policy import V1beta1NetworkPolicy
from .v1beta1_network_policy_egress_rule import V1beta1NetworkPolicyEgressRule
from .v1beta1_network_policy_ingress_rule import V1beta1NetworkPolicyIngressRule
from .v1beta1_network_policy_list import V1beta1NetworkPolicyList
from .v1beta1_network_policy_peer import V1beta1NetworkPolicyPeer
from .v1beta1_network_policy_port import V1beta1NetworkPolicyPort
from .v1beta1_network_policy_spec import V1beta1NetworkPolicySpec
from .v1beta1_non_resource_attributes import V1beta1NonResourceAttributes
from .v1beta1_non_resource_rule import V1beta1NonResourceRule
from .v1beta1_pod_disruption_budget import V1beta1PodDisruptionBudget
from .v1beta1_pod_disruption_budget_list import V1beta1PodDisruptionBudgetList
from .v1beta1_pod_disruption_budget_spec import V1beta1PodDisruptionBudgetSpec
from .v1beta1_pod_disruption_budget_status import V1beta1PodDisruptionBudgetStatus
from .v1beta1_policy_rule import V1beta1PolicyRule
from .v1beta1_priority_class import V1beta1PriorityClass
from .v1beta1_priority_class_list import V1beta1PriorityClassList
from .v1beta1_replica_set import V1beta1ReplicaSet
from .v1beta1_replica_set_condition import V1beta1ReplicaSetCondition
from .v1beta1_replica_set_list import V1beta1ReplicaSetList
from .v1beta1_replica_set_spec import V1beta1ReplicaSetSpec
from .v1beta1_replica_set_status import V1beta1ReplicaSetStatus
from .v1beta1_resource_attributes import V1beta1ResourceAttributes
from .v1beta1_resource_rule import V1beta1ResourceRule
from .v1beta1_role import V1beta1Role
from .v1beta1_role_binding import V1beta1RoleBinding
from .v1beta1_role_binding_list import V1beta1RoleBindingList
from .v1beta1_role_list import V1beta1RoleList
from .v1beta1_role_ref import V1beta1RoleRef
from .v1beta1_rolling_update_daemon_set import V1beta1RollingUpdateDaemonSet
from .v1beta1_rolling_update_stateful_set_strategy import V1beta1RollingUpdateStatefulSetStrategy
from .v1beta1_rule_with_operations import V1beta1RuleWithOperations
from .v1beta1_runtime_class import V1beta1RuntimeClass
from .v1beta1_runtime_class_list import V1beta1RuntimeClassList
from .v1beta1_self_subject_access_review import V1beta1SelfSubjectAccessReview
from .v1beta1_self_subject_access_review_spec import V1beta1SelfSubjectAccessReviewSpec
from .v1beta1_self_subject_rules_review import V1beta1SelfSubjectRulesReview
from .v1beta1_self_subject_rules_review_spec import V1beta1SelfSubjectRulesReviewSpec
from .v1beta1_stateful_set import V1beta1StatefulSet
from .v1beta1_stateful_set_condition import V1beta1StatefulSetCondition
from .v1beta1_stateful_set_list import V1beta1StatefulSetList
from .v1beta1_stateful_set_spec import V1beta1StatefulSetSpec
from .v1beta1_stateful_set_status import V1beta1StatefulSetStatus
from .v1beta1_stateful_set_update_strategy import V1beta1StatefulSetUpdateStrategy
from .v1beta1_storage_class import V1beta1StorageClass
from .v1beta1_storage_class_list import V1beta1StorageClassList
from .v1beta1_subject import V1beta1Subject
from .v1beta1_subject_access_review import V1beta1SubjectAccessReview
from .v1beta1_subject_access_review_spec import V1beta1SubjectAccessReviewSpec
from .v1beta1_subject_access_review_status import V1beta1SubjectAccessReviewStatus
from .v1beta1_subject_rules_review_status import V1beta1SubjectRulesReviewStatus
from .v1beta1_token_review import V1beta1TokenReview
from .v1beta1_token_review_spec import V1beta1TokenReviewSpec
from .v1beta1_token_review_status import V1beta1TokenReviewStatus
from .v1beta1_user_info import V1beta1UserInfo
from .v1beta1_validating_webhook_configuration import V1beta1ValidatingWebhookConfiguration
from .v1beta1_validating_webhook_configuration_list import V1beta1ValidatingWebhookConfigurationList
from .v1beta1_volume_attachment import V1beta1VolumeAttachment
from .v1beta1_volume_attachment_list import V1beta1VolumeAttachmentList
from .v1beta1_volume_attachment_source import V1beta1VolumeAttachmentSource
from .v1beta1_volume_attachment_spec import V1beta1VolumeAttachmentSpec
from .v1beta1_volume_attachment_status import V1beta1VolumeAttachmentStatus
from .v1beta1_volume_error import V1beta1VolumeError
from .v1beta1_webhook import V1beta1Webhook
from .v1beta2_controller_revision import V1beta2ControllerRevision
from .v1beta2_controller_revision_list import V1beta2ControllerRevisionList
from .v1beta2_daemon_set import V1beta2DaemonSet
from .v1beta2_daemon_set_condition import V1beta2DaemonSetCondition
from .v1beta2_daemon_set_list import V1beta2DaemonSetList
from .v1beta2_daemon_set_spec import V1beta2DaemonSetSpec
from .v1beta2_daemon_set_status import V1beta2DaemonSetStatus
from .v1beta2_daemon_set_update_strategy import V1beta2DaemonSetUpdateStrategy
from .v1beta2_deployment import V1beta2Deployment
from .v1beta2_deployment_condition import V1beta2DeploymentCondition
from .v1beta2_deployment_list import V1beta2DeploymentList
from .v1beta2_deployment_spec import V1beta2DeploymentSpec
from .v1beta2_deployment_status import V1beta2DeploymentStatus
from .v1beta2_deployment_strategy import V1beta2DeploymentStrategy
from .v1beta2_replica_set import V1beta2ReplicaSet
from .v1beta2_replica_set_condition import V1beta2ReplicaSetCondition
from .v1beta2_replica_set_list import V1beta2ReplicaSetList
from .v1beta2_replica_set_spec import V1beta2ReplicaSetSpec
from .v1beta2_replica_set_status import V1beta2ReplicaSetStatus
from .v1beta2_rolling_update_daemon_set import V1beta2RollingUpdateDaemonSet
from .v1beta2_rolling_update_deployment import V1beta2RollingUpdateDeployment
from .v1beta2_rolling_update_stateful_set_strategy import V1beta2RollingUpdateStatefulSetStrategy
from .v1beta2_scale import V1beta2Scale
from .v1beta2_scale_spec import V1beta2ScaleSpec
from .v1beta2_scale_status import V1beta2ScaleStatus
from .v1beta2_stateful_set import V1beta2StatefulSet
from .v1beta2_stateful_set_condition import V1beta2StatefulSetCondition
from .v1beta2_stateful_set_list import V1beta2StatefulSetList
from .v1beta2_stateful_set_spec import V1beta2StatefulSetSpec
from .v1beta2_stateful_set_status import V1beta2StatefulSetStatus
from .v1beta2_stateful_set_update_strategy import V1beta2StatefulSetUpdateStrategy
from .v2alpha1_cron_job import V2alpha1CronJob
from .v2alpha1_cron_job_list import V2alpha1CronJobList
from .v2alpha1_cron_job_spec import V2alpha1CronJobSpec
from .v2alpha1_cron_job_status import V2alpha1CronJobStatus
from .v2alpha1_job_template_spec import V2alpha1JobTemplateSpec
from .v2beta1_cross_version_object_reference import V2beta1CrossVersionObjectReference
from .v2beta1_external_metric_source import V2beta1ExternalMetricSource
from .v2beta1_external_metric_status import V2beta1ExternalMetricStatus
from .v2beta1_horizontal_pod_autoscaler import V2beta1HorizontalPodAutoscaler
from .v2beta1_horizontal_pod_autoscaler_condition import V2beta1HorizontalPodAutoscalerCondition
from .v2beta1_horizontal_pod_autoscaler_list import V2beta1HorizontalPodAutoscalerList
from .v2beta1_horizontal_pod_autoscaler_spec import V2beta1HorizontalPodAutoscalerSpec
from .v2beta1_horizontal_pod_autoscaler_status import V2beta1HorizontalPodAutoscalerStatus
from .v2beta1_metric_spec import V2beta1MetricSpec
from .v2beta1_metric_status import V2beta1MetricStatus
from .v2beta1_object_metric_source import V2beta1ObjectMetricSource
from .v2beta1_object_metric_status import V2beta1ObjectMetricStatus
from .v2beta1_pods_metric_source import V2beta1PodsMetricSource
from .v2beta1_pods_metric_status import V2beta1PodsMetricStatus
from .v2beta1_resource_metric_source import V2beta1ResourceMetricSource
from .v2beta1_resource_metric_status import V2beta1ResourceMetricStatus
from .v2beta2_cross_version_object_reference import V2beta2CrossVersionObjectReference
from .v2beta2_external_metric_source import V2beta2ExternalMetricSource
from .v2beta2_external_metric_status import V2beta2ExternalMetricStatus
from .v2beta2_horizontal_pod_autoscaler import V2beta2HorizontalPodAutoscaler
from .v2beta2_horizontal_pod_autoscaler_condition import V2beta2HorizontalPodAutoscalerCondition
from .v2beta2_horizontal_pod_autoscaler_list import V2beta2HorizontalPodAutoscalerList
from .v2beta2_horizontal_pod_autoscaler_spec import V2beta2HorizontalPodAutoscalerSpec
from .v2beta2_horizontal_pod_autoscaler_status import V2beta2HorizontalPodAutoscalerStatus
from .v2beta2_metric_identifier import V2beta2MetricIdentifier
from .v2beta2_metric_spec import V2beta2MetricSpec
from .v2beta2_metric_status import V2beta2MetricStatus
from .v2beta2_metric_target import V2beta2MetricTarget
from .v2beta2_metric_value_status import V2beta2MetricValueStatus
from .v2beta2_object_metric_source import V2beta2ObjectMetricSource
from .v2beta2_object_metric_status import V2beta2ObjectMetricStatus
from .v2beta2_pods_metric_source import V2beta2PodsMetricSource
from .v2beta2_pods_metric_status import V2beta2PodsMetricStatus
from .v2beta2_resource_metric_source import V2beta2ResourceMetricSource
from .v2beta2_resource_metric_status import V2beta2ResourceMetricStatus
from .version_info import VersionInfo



# -*- coding: utf-8 -*-
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from .services.config_service_v2 import ConfigServiceV2Client
from .services.config_service_v2 import ConfigServiceV2AsyncClient
from .services.logging_service_v2 import LoggingServiceV2Client
from .services.logging_service_v2 import LoggingServiceV2AsyncClient
from .services.metrics_service_v2 import MetricsServiceV2Client
from .services.metrics_service_v2 import MetricsServiceV2AsyncClient

from .types.log_entry import LogEntry
from .types.log_entry import LogEntryOperation
from .types.log_entry import LogEntrySourceLocation
from .types.log_entry import LogSplit
from .types.logging import DeleteLogRequest
from .types.logging import ListLogEntriesRequest
from .types.logging import ListLogEntriesResponse
from .types.logging import ListLogsRequest
from .types.logging import ListLogsResponse
from .types.logging import ListMonitoredResourceDescriptorsRequest
from .types.logging import ListMonitoredResourceDescriptorsResponse
from .types.logging import TailLogEntriesRequest
from .types.logging import TailLogEntriesResponse
from .types.logging import WriteLogEntriesPartialErrors
from .types.logging import WriteLogEntriesRequest
from .types.logging import WriteLogEntriesResponse
from .types.logging_config import BigQueryOptions
from .types.logging_config import CmekSettings
from .types.logging_config import CopyLogEntriesMetadata
from .types.logging_config import CopyLogEntriesRequest
from .types.logging_config import CopyLogEntriesResponse
from .types.logging_config import CreateBucketRequest
from .types.logging_config import CreateExclusionRequest
from .types.logging_config import CreateSinkRequest
from .types.logging_config import CreateViewRequest
from .types.logging_config import DeleteBucketRequest
from .types.logging_config import DeleteExclusionRequest
from .types.logging_config import DeleteSinkRequest
from .types.logging_config import DeleteViewRequest
from .types.logging_config import GetBucketRequest
from .types.logging_config import GetCmekSettingsRequest
from .types.logging_config import GetExclusionRequest
from .types.logging_config import GetSettingsRequest
from .types.logging_config import GetSinkRequest
from .types.logging_config import GetViewRequest
from .types.logging_config import ListBucketsRequest
from .types.logging_config import ListBucketsResponse
from .types.logging_config import ListExclusionsRequest
from .types.logging_config import ListExclusionsResponse
from .types.logging_config import ListSinksRequest
from .types.logging_config import ListSinksResponse
from .types.logging_config import ListViewsRequest
from .types.logging_config import ListViewsResponse
from .types.logging_config import LogBucket
from .types.logging_config import LogExclusion
from .types.logging_config import LogSink
from .types.logging_config import LogView
from .types.logging_config import Settings
from .types.logging_config import UndeleteBucketRequest
from .types.logging_config import UpdateBucketRequest
from .types.logging_config import UpdateCmekSettingsRequest
from .types.logging_config import UpdateExclusionRequest
from .types.logging_config import UpdateSettingsRequest
from .types.logging_config import UpdateSinkRequest
from .types.logging_config import UpdateViewRequest
from .types.logging_config import LifecycleState
from .types.logging_config import OperationState
from .types.logging_metrics import CreateLogMetricRequest
from .types.logging_metrics import DeleteLogMetricRequest
from .types.logging_metrics import GetLogMetricRequest
from .types.logging_metrics import ListLogMetricsRequest
from .types.logging_metrics import ListLogMetricsResponse
from .types.logging_metrics import LogMetric
from .types.logging_metrics import UpdateLogMetricRequest

__all__ = (
    'ConfigServiceV2AsyncClient',
    'LoggingServiceV2AsyncClient',
    'MetricsServiceV2AsyncClient',
'BigQueryOptions',
'CmekSettings',
'ConfigServiceV2Client',
'CopyLogEntriesMetadata',
'CopyLogEntriesRequest',
'CopyLogEntriesResponse',
'CreateBucketRequest',
'CreateExclusionRequest',
'CreateLogMetricRequest',
'CreateSinkRequest',
'CreateViewRequest',
'DeleteBucketRequest',
'DeleteExclusionRequest',
'DeleteLogMetricRequest',
'DeleteLogRequest',
'DeleteSinkRequest',
'DeleteViewRequest',
'GetBucketRequest',
'GetCmekSettingsRequest',
'GetExclusionRequest',
'GetLogMetricRequest',
'GetSettingsRequest',
'GetSinkRequest',
'GetViewRequest',
'LifecycleState',
'ListBucketsRequest',
'ListBucketsResponse',
'ListExclusionsRequest',
'ListExclusionsResponse',
'ListLogEntriesRequest',
'ListLogEntriesResponse',
'ListLogMetricsRequest',
'ListLogMetricsResponse',
'ListLogsRequest',
'ListLogsResponse',
'ListMonitoredResourceDescriptorsRequest',
'ListMonitoredResourceDescriptorsResponse',
'ListSinksRequest',
'ListSinksResponse',
'ListViewsRequest',
'ListViewsResponse',
'LogBucket',
'LogEntry',
'LogEntryOperation',
'LogEntrySourceLocation',
'LogExclusion',
'LogMetric',
'LogSink',
'LogSplit',
'LogView',
'LoggingServiceV2Client',
'MetricsServiceV2Client',
'OperationState',
'Settings',
'TailLogEntriesRequest',
'TailLogEntriesResponse',
'UndeleteBucketRequest',
'UpdateBucketRequest',
'UpdateCmekSettingsRequest',
'UpdateExclusionRequest',
'UpdateLogMetricRequest',
'UpdateSettingsRequest',
'UpdateSinkRequest',
'UpdateViewRequest',
'WriteLogEntriesPartialErrors',
'WriteLogEntriesRequest',
'WriteLogEntriesResponse',
)



# -*- coding: utf-8 -*- #
# Copyright 2016 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""The configs waiters create command."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.runtime_config import util
from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.runtime_config import flags
from googlecloudsdk.core import log


class Create(base.CreateCommand):
  """Create waiter resources.

  This command creates a new waiter resource with the specified name and
  parameters.
  """

  detailed_help = {
      'EXAMPLES': """
          To create a waiter in "my-config" with success and failure paths
          nested under "/status", run:

            $ {command} my-waiter --config-name=my-config --timeout=15m --success-cardinality-path=/status/success --success-cardinality-number=5 --failure-cardinality-path=/status/failure --failure-cardinality-number=1

          This waiter will wait for at most 15 minutes for the first of two
          possible scenarios: 1) five or more variables are written to the
          /status/success/ path; or 2) one or more variables are written to the
          /status/failure/ path.

          To create a waiter without a failure path, run:

            $ {command} my-waiter --config-name=my-config --timeout=15m --success-cardinality-path=/status/success --success-cardinality-number=5

          This waiter will wait until 5 or more success variables are written,
          or the 15 minute timeout elapses.
          """,
  }

  @staticmethod
  def Args(parser):
    """Args is called by calliope to gather arguments for this command.

    Args:
      parser: An argparse parser that you can use to add arguments that go
          on the command line after this command. Positional arguments are
          allowed.
    """
    flags.AddRequiredConfigFlag(parser)
    base.ASYNC_FLAG.AddToParser(parser)

    parser.add_argument(
        '--timeout',
        type=arg_parsers.Duration(lower_bound='1s',
                                  upper_bound='{0}s'.format(
                                      util.MAX_WAITER_TIMEOUT)),
        required=True,
        help="""\
        The amount of time to wait before failing with DEADLINE_EXCEEDED.
        See $ gcloud topic datetimes for information on duration formats.
        """)

    parser.add_argument(
        '--success-cardinality-path',
        help='The path where success variables are written.',
        required=True)
    parser.add_argument(
        '--success-cardinality-number',
        help='The minimum required number of success variables.',
        type=arg_parsers.BoundedInt(lower_bound=1),
        default=1)

    parser.add_argument(
        '--failure-cardinality-path',
        help='The path where failure variables are written.')
    parser.add_argument(
        '--failure-cardinality-number',
        help='The minimum required number of failure variables.',
        type=arg_parsers.BoundedInt(lower_bound=1),
        default=1)

    parser.add_argument('name', help='The waiter name.')

  def Run(self, args):
    """Run 'runtime-configs waiters create'.

    Args:
      args: argparse.Namespace, The arguments that this command was invoked
          with.

    Returns:
      The associated waiter operation.

    Raises:
      HttpException: An http error response was received while executing api
          request.
    """
    waiter_client = util.WaiterClient()
    messages = util.Messages()

    waiter_resource = util.ParseWaiterName(args.name, args)
    project = waiter_resource.projectsId
    config = waiter_resource.configsId

    success = messages.EndCondition(
        cardinality=messages.Cardinality(
            path=args.success_cardinality_path,
            number=args.success_cardinality_number,
        )
    )

    if args.failure_cardinality_path:
      failure = messages.EndCondition(
          cardinality=messages.Cardinality(
              path=args.failure_cardinality_path,
              number=args.failure_cardinality_number,
          )
      )
    else:
      failure = None

    result = waiter_client.Create(
        messages.RuntimeconfigProjectsConfigsWaitersCreateRequest(
            parent=util.ConfigPath(project, config),
            waiter=messages.Waiter(
                name=waiter_resource.RelativeName(),
                timeout='{0}s'.format(args.timeout),
                success=success,
                failure=failure,
            )
        )
    )

    log.CreatedResource(waiter_resource)

    if args.async_:
      # In async mode, we return the current waiter representation.
      # The waiter resource exists immediately after creation; the
      # operation resource returned from CreateWaiter only tracks the
      # waiting process.
      self._async_resource = waiter_resource
      request = (waiter_client.client.MESSAGES_MODULE
                 .RuntimeconfigProjectsConfigsWaitersGetRequest(
                     name=waiter_resource.RelativeName()))
      result = waiter_client.Get(request)
    else:
      self._async_resource = None
      result = util.WaitForWaiter(waiter_resource)
      if util.IsFailedWaiter(result):
        self.exit_code = 2  # exit with code 2 if the result waiter failed.

    return util.FormatWaiter(result)

  def Epilog(self, unused_resources_were_displayed):
    """Called after resources are displayed if the default format was used.

    Args:
      unused_resources_were_displayed: Unused.
    """
    if self._async_resource:
      log.status.Print()
      log.status.Print(
          ('The wait command can be used to monitor the progress '
           'of waiter [{0}].').format(self._async_resource.Name()))



# Lint as: python3
# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""`gcloud alpha scc settings services modules disable` command."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.scc.settings import flags
from googlecloudsdk.command_lib.scc.settings import utils


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class Disable(base.UpdateCommand):
  """Disable a module in Security Command Center(SCC)."""

  detailed_help = {
      'DESCRIPTION':
          """\
          Disable a module in Security Command Center(SCC).
          """,
      'EXAMPLES':
          """\
          To disable the "PUBLIC_BUCKET_ACL" module in service SECURITY_HEALTH_ANALYTICS of organization "12345", run:

            $ {command} --organization=12345  --service=SECURITY_HEALTH_ANALYTICS --module=PUBLIC_BUCKET_ACL
      """
  }

  @staticmethod
  def Args(parser):
    flags.ExtractRequiredFlags(parser)
    flags.AddServiceArgument(parser)
    flags.AddModuleArgument(parser)

  def Run(self, args):
    """Call corresponding APIs based on the flag."""
    return utils.SettingsClient().DisableModule(args)



# Lint as: python3
# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Restore a subordinate Certificate Authority."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.privateca import base as privateca_base
from googlecloudsdk.api_lib.privateca import request_utils
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.privateca import operations
from googlecloudsdk.command_lib.privateca import resource_args
from googlecloudsdk.core import log


@base.ReleaseTracks(base.ReleaseTrack.BETA)
class Restore(base.SilentCommand):
  r"""Restore a subordinate Certificate Authority.

    Restores a subordinate Certificate Authority that has been marked for
    deletion. A Certificate Authority can be restored within 30 days of being
    scheduled for deletion. Use this command to halt the deletion process. A
    restored CA will move to DISABLED state.

    ## EXAMPLES

    To restore a subordinate CA:

        $ {command} server-tls-1 --location=us-west1
  """

  @staticmethod
  def Args(parser):
    resource_args.AddCertificateAuthorityPositionalResourceArg(
        parser, 'to restore')

  def Run(self, args):
    client = privateca_base.GetClientInstance()
    messages = privateca_base.GetMessagesModule()

    ca_ref = args.CONCEPTS.certificate_authority.Parse()

    current_ca = client.projects_locations_certificateAuthorities.Get(
        messages.PrivatecaProjectsLocationsCertificateAuthoritiesGetRequest(
            name=ca_ref.RelativeName()))

    resource_args.CheckExpectedCAType(
        messages.CertificateAuthority.TypeValueValuesEnum.SUBORDINATE,
        current_ca)

    operation = client.projects_locations_certificateAuthorities.Restore(
        messages.PrivatecaProjectsLocationsCertificateAuthoritiesRestoreRequest(
            name=ca_ref.RelativeName(),
            restoreCertificateAuthorityRequest=messages
            .RestoreCertificateAuthorityRequest(
                requestId=request_utils.GenerateRequestId())))

    operations.Await(operation, 'Restoring Subordinate CA')

    log.status.Print('Restored Subordinate CA [{}].'.format(
        ca_ref.RelativeName()))



# Copyright 2016 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# -*- coding: utf-8 -*- #
"""Directly processes text of cron.xml.

CronXmlParser is called with an XML string to produce a CronXml object
containing the data from the XML.

CronXmlParser: converts XML to CronXml objct
Cron: describes a single cron specified in cron.xml
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from xml.etree import ElementTree

from googlecloudsdk.third_party.appengine.tools import xml_parser_utils
from googlecloudsdk.third_party.appengine.tools.app_engine_config_exception import AppEngineConfigException

# pylint: disable=g-import-not-at-top
from googlecloudsdk.third_party.appengine._internal import six_subset

# groc depends on antlr3 which is py2-only, so conditionally import based on
# python version. See comments under GrocValidator.Validate for more context.
if six_subset.PY2:
  from googlecloudsdk.third_party.appengine.googlecron import groc
  from googlecloudsdk.third_party.appengine.googlecron import groctimespecification
else:
  groc = None
  groctimespecification = None
# pylint: enable=g-import-not-at-top

_RETRY_PARAMETER_TAGS = ('job-retry-limit',
                         'job-age-limit',
                         'min-backoff-seconds',
                         'max-backoff-seconds',
                         'max-doublings')


def GetCronYaml(unused_application, cron_xml_str):
  return _MakeCronListIntoYaml(CronXmlParser().ProcessXml(cron_xml_str))


def _MakeCronListIntoYaml(cron_list):
  """Converts list of yaml statements describing cron jobs into a string."""
  statements = ['cron:']
  for cron in cron_list:
    statements += cron.ToYaml()
  return '\n'.join(statements) + '\n'


def _ProcessRetryParametersNode(node, cron):
  """Converts <retry-parameters> in node to cron.retry_parameters."""

  retry_parameters_node = xml_parser_utils.GetChild(node, 'retry-parameters')
  if retry_parameters_node is None:
    cron.retry_parameters = None
    return

  retry_parameters = _RetryParameters()
  cron.retry_parameters = retry_parameters
  for tag in _RETRY_PARAMETER_TAGS:
    if xml_parser_utils.GetChild(retry_parameters_node, tag) is not None:
      setattr(
          retry_parameters,
          tag.replace('-', '_'),
          xml_parser_utils.GetChildNodeText(retry_parameters_node, tag))


class CronXmlParser(object):
  """Provides logic for walking down XML tree and pulling data."""

  def ProcessXml(self, xml_str):
    """Parses XML string and returns object representation of relevant info.

    Args:
      xml_str: The XML string.
    Returns:
      A list of Cron objects containing information about cron jobs from the
      XML.
    Raises:
      AppEngineConfigException: In case of malformed XML or illegal inputs.
    """

    try:
      self.crons = []
      self.errors = []
      xml_root = ElementTree.fromstring(xml_str)
      if xml_root.tag != 'cronentries':
        raise AppEngineConfigException('Root tag must be <cronentries>')

      for child in xml_root.getchildren():
        self.ProcessCronNode(child)

      if self.errors:
        raise AppEngineConfigException('\n'.join(self.errors))

      return self.crons
    except ElementTree.ParseError:
      raise AppEngineConfigException('Bad input -- not valid XML')

  def ProcessCronNode(self, node):
    """Processes XML <cron> nodes into Cron objects.

    The following information is parsed out:
      description: Describing the purpose of the cron job.
      url: The location of the script.
      schedule: Written in groc; the schedule according to which the job is
        executed.
      timezone: The timezone that the schedule runs in.
      target: Which version of the app this applies to.

    Args:
      node: <cron> XML node in cron.xml.
    """
    tag = xml_parser_utils.GetTag(node)
    if tag != 'cron':
      self.errors.append('Unrecognized node: <%s>' % tag)
      return

    cron = Cron()
    cron.url = xml_parser_utils.GetChildNodeText(node, 'url')
    cron.timezone = xml_parser_utils.GetChildNodeText(node, 'timezone')
    cron.target = xml_parser_utils.GetChildNodeText(node, 'target')
    cron.description = xml_parser_utils.GetChildNodeText(node, 'description')
    cron.schedule = xml_parser_utils.GetChildNodeText(node, 'schedule')
    _ProcessRetryParametersNode(node, cron)

    validation_error = self._ValidateCronEntry(cron)
    if validation_error:
      self.errors.append(validation_error)
    else:
      self.crons.append(cron)

  def _ValidateCronEntry(self, cron):
    # TODO(user): consider timezone validation, not currently done by the
    # Java SDK.
    if not cron.url:
      return 'No URL for <cron> entry'
    if not cron.schedule:
      return "No schedule provided for <cron> entry with URL '%s'" % cron.url
    # If we're running on py3 and don't have access to groctimespecification,
    # then the server will still do the validation on the schedule property.
    if groc and groctimespecification:
      try:
        groctimespecification.GrocTimeSpecification(cron.schedule)
      except groc.GrocException:
        return ("Text '%s' in <schedule> node failed to parse,"
                ' for <cron> entry with url %s.'
                % (cron.schedule, cron.url))


class _RetryParameters(object):
  """Object that contains retry xml tags converted to object attributes."""

  def GetYamlStatementsList(self):
    """Converts retry parameter fields to a YAML statement list."""

    tag_statements = []
    field_names = (tag.replace('-', '_') for tag in _RETRY_PARAMETER_TAGS)
    for field in field_names:
      field_value = getattr(self, field, None)
      if field_value:
        tag_statements.append('    %s: %s' % (field, field_value))

    if not tag_statements:
      return ['  retry_parameters: {}']
    return ['  retry_parameters:'] + tag_statements


class Cron(object):
  """Instances contain information about individual cron entries."""
  TZ_GMT = 'UTC'

  def ToYaml(self):
    """Returns data from Cron object as a list of Yaml statements."""
    statements = [
        '- url: %s' % self._SanitizeForYaml(self.url),
        '  schedule: %s' % self._SanitizeForYaml(self.schedule)]
    for optional in ('target', 'timezone', 'description'):
      field = getattr(self, optional)
      if field:
        statements.append('  %s: %s' % (optional, self._SanitizeForYaml(field)))
    retry_parameters = getattr(self, 'retry_parameters', None)
    if retry_parameters:
      statements += retry_parameters.GetYamlStatementsList()
    return statements

  def _SanitizeForYaml(self, field):
    return "'%s'" % field.replace('\n', ' ').replace("'", "''")



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Surface for creating domain mappings."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.run import connection_context
from googlecloudsdk.command_lib.run import exceptions
from googlecloudsdk.command_lib.run import flags
from googlecloudsdk.command_lib.run import platforms
from googlecloudsdk.command_lib.run import resource_args
from googlecloudsdk.command_lib.run import serverless_operations
from googlecloudsdk.command_lib.util.concepts import concept_parsers
from googlecloudsdk.command_lib.util.concepts import presentation_specs


@base.ReleaseTracks(base.ReleaseTrack.GA)
class Describe(base.Command):
  """Describe domain mappings for Cloud Run for Anthos."""

  detailed_help = {
      'DESCRIPTION':
          """\
          {description}

          For domain mapping support with fully managed Cloud Run, use
          `gcloud beta run domain-mappings describe`.""",
      'EXAMPLES':
          """\
          To describe a Cloud Run domain mapping, run:

              $ {command} --domain=www.example.com
          """,
  }

  @staticmethod
  def CommonArgs(parser):
    domain_mapping_presentation = presentation_specs.ResourcePresentationSpec(
        '--domain',
        resource_args.GetDomainMappingResourceSpec(),
        'Domain name is the ID of DomainMapping resource.',
        required=True,
        prefixes=False)
    concept_parsers.ConceptParser([
        domain_mapping_presentation]).AddToParser(parser)

    parser.display_info.AddFormat('yaml')

  @staticmethod
  def Args(parser):
    Describe.CommonArgs(parser)

  def Run(self, args):
    """Describe a domain mapping."""
    # domains.cloudrun.com api group only supports v1alpha1 on clusters.
    conn_context = connection_context.GetConnectionContext(
        args,
        flags.Product.RUN,
        self.ReleaseTrack(),
        version_override=('v1alpha1' if
                          platforms.GetPlatform() != platforms.PLATFORM_MANAGED
                          else None))
    domain_mapping_ref = args.CONCEPTS.domain.Parse()
    with serverless_operations.Connect(conn_context) as client:
      domain_mapping = client.GetDomainMapping(domain_mapping_ref)
      if not domain_mapping:
        raise exceptions.ArgumentError(
            'Cannot find domain mapping for domain name [{}]'.format(
                domain_mapping_ref.domainmappingsId))
      return domain_mapping


@base.ReleaseTracks(base.ReleaseTrack.BETA)
class BetaDescribe(Describe):
  """Describe domain mappings."""

  detailed_help = {
      'DESCRIPTION':
          '{description}',
      'EXAMPLES':
          """\
          To describe a Cloud Run domain mapping, run:

              $ {command} --domain=www.example.com
          """,
  }

  @staticmethod
  def Args(parser):
    Describe.CommonArgs(parser)


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class AlphaDescribe(BetaDescribe):
  """Describe domain mappings."""

  @staticmethod
  def Args(parser):
    Describe.CommonArgs(parser)



# -*- coding: utf-8 -*- #
# Copyright 2015 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Resource definitions for cloud platform apis."""

import enum


BASE_URL = 'https://recommender.googleapis.com/v1beta1/'
DOCS_URL = 'https://cloud.google.com/recommender/docs/'


class Collections(enum.Enum):
  """Collections for all supported apis."""

  BILLINGACCOUNTS = (
      'billingAccounts',
      'billingAccounts/{billingAccountsId}',
      {},
      ['billingAccountsId'],
      True
  )
  BILLINGACCOUNTS_LOCATIONS = (
      'billingAccounts.locations',
      'billingAccounts/{billingAccountsId}/locations/{locationsId}',
      {},
      ['billingAccountsId', 'locationsId'],
      True
  )
  BILLINGACCOUNTS_LOCATIONS_INSIGHTTYPES = (
      'billingAccounts.locations.insightTypes',
      'billingAccounts/{billingAccountsId}/locations/{locationsId}/'
      'insightTypes/{insightTypesId}',
      {},
      ['billingAccountsId', 'locationsId', 'insightTypesId'],
      True
  )
  BILLINGACCOUNTS_LOCATIONS_INSIGHTTYPES_INSIGHTS = (
      'billingAccounts.locations.insightTypes.insights',
      '{+name}',
      {
          '':
              'billingAccounts/{billingAccountsId}/locations/{locationsId}/'
              'insightTypes/{insightTypesId}/insights/{insightsId}',
      },
      ['name'],
      True
  )
  BILLINGACCOUNTS_LOCATIONS_RECOMMENDERS = (
      'billingAccounts.locations.recommenders',
      'billingAccounts/{billingAccountsId}/locations/{locationsId}/'
      'recommenders/{recommendersId}',
      {},
      ['billingAccountsId', 'locationsId', 'recommendersId'],
      True
  )
  BILLINGACCOUNTS_LOCATIONS_RECOMMENDERS_RECOMMENDATIONS = (
      'billingAccounts.locations.recommenders.recommendations',
      '{+name}',
      {
          '':
              'billingAccounts/{billingAccountsId}/locations/{locationsId}/'
              'recommenders/{recommendersId}/recommendations/'
              '{recommendationsId}',
      },
      ['name'],
      True
  )
  FOLDERS = (
      'folders',
      'folders/{foldersId}',
      {},
      ['foldersId'],
      True
  )
  FOLDERS_LOCATIONS = (
      'folders.locations',
      'folders/{foldersId}/locations/{locationsId}',
      {},
      ['foldersId', 'locationsId'],
      True
  )
  FOLDERS_LOCATIONS_INSIGHTTYPES = (
      'folders.locations.insightTypes',
      'folders/{foldersId}/locations/{locationsId}/insightTypes/'
      '{insightTypesId}',
      {},
      ['foldersId', 'locationsId', 'insightTypesId'],
      True
  )
  FOLDERS_LOCATIONS_INSIGHTTYPES_INSIGHTS = (
      'folders.locations.insightTypes.insights',
      '{+name}',
      {
          '':
              'folders/{foldersId}/locations/{locationsId}/insightTypes/'
              '{insightTypesId}/insights/{insightsId}',
      },
      ['name'],
      True
  )
  FOLDERS_LOCATIONS_RECOMMENDERS = (
      'folders.locations.recommenders',
      'folders/{foldersId}/locations/{locationsId}/recommenders/'
      '{recommendersId}',
      {},
      ['foldersId', 'locationsId', 'recommendersId'],
      True
  )
  FOLDERS_LOCATIONS_RECOMMENDERS_RECOMMENDATIONS = (
      'folders.locations.recommenders.recommendations',
      '{+name}',
      {
          '':
              'folders/{foldersId}/locations/{locationsId}/recommenders/'
              '{recommendersId}/recommendations/{recommendationsId}',
      },
      ['name'],
      True
  )
  ORGANIZATIONS = (
      'organizations',
      'organizations/{organizationsId}',
      {},
      ['organizationsId'],
      True
  )
  ORGANIZATIONS_LOCATIONS = (
      'organizations.locations',
      'organizations/{organizationsId}/locations/{locationsId}',
      {},
      ['organizationsId', 'locationsId'],
      True
  )
  ORGANIZATIONS_LOCATIONS_INSIGHTTYPES = (
      'organizations.locations.insightTypes',
      'organizations/{organizationsId}/locations/{locationsId}/insightTypes/'
      '{insightTypesId}',
      {},
      ['organizationsId', 'locationsId', 'insightTypesId'],
      True
  )
  ORGANIZATIONS_LOCATIONS_INSIGHTTYPES_INSIGHTS = (
      'organizations.locations.insightTypes.insights',
      '{+name}',
      {
          '':
              'organizations/{organizationsId}/locations/{locationsId}/'
              'insightTypes/{insightTypesId}/insights/{insightsId}',
      },
      ['name'],
      True
  )
  ORGANIZATIONS_LOCATIONS_RECOMMENDERS = (
      'organizations.locations.recommenders',
      'organizations/{organizationsId}/locations/{locationsId}/recommenders/'
      '{recommendersId}',
      {},
      ['organizationsId', 'locationsId', 'recommendersId'],
      True
  )
  ORGANIZATIONS_LOCATIONS_RECOMMENDERS_RECOMMENDATIONS = (
      'organizations.locations.recommenders.recommendations',
      '{+name}',
      {
          '':
              'organizations/{organizationsId}/locations/{locationsId}/'
              'recommenders/{recommendersId}/recommendations/'
              '{recommendationsId}',
      },
      ['name'],
      True
  )
  PROJECTS = (
      'projects',
      'projects/{projectsId}',
      {},
      ['projectsId'],
      True
  )
  PROJECTS_LOCATIONS = (
      'projects.locations',
      'projects/{projectsId}/locations/{locationsId}',
      {},
      ['projectsId', 'locationsId'],
      True
  )
  PROJECTS_LOCATIONS_INSIGHTTYPES = (
      'projects.locations.insightTypes',
      'projects/{projectsId}/locations/{locationsId}/insightTypes/'
      '{insightTypesId}',
      {},
      ['projectsId', 'locationsId', 'insightTypesId'],
      True
  )
  PROJECTS_LOCATIONS_INSIGHTTYPES_INSIGHTS = (
      'projects.locations.insightTypes.insights',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/insightTypes/'
              '{insightTypesId}/insights/{insightsId}',
      },
      ['name'],
      True
  )
  PROJECTS_LOCATIONS_RECOMMENDERS = (
      'projects.locations.recommenders',
      'projects/{projectsId}/locations/{locationsId}/recommenders/'
      '{recommendersId}',
      {},
      ['projectsId', 'locationsId', 'recommendersId'],
      True
  )
  PROJECTS_LOCATIONS_RECOMMENDERS_RECOMMENDATIONS = (
      'projects.locations.recommenders.recommendations',
      '{+name}',
      {
          '':
              'projects/{projectsId}/locations/{locationsId}/recommenders/'
              '{recommendersId}/recommendations/{recommendationsId}',
      },
      ['name'],
      True
  )

  def __init__(self, collection_name, path, flat_paths, params,
               enable_uri_parsing):
    self.collection_name = collection_name
    self.path = path
    self.flat_paths = flat_paths
    self.params = params
    self.enable_uri_parsing = enable_uri_parsing



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""`gcloud iot registries credentials create` command."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.cloudiot import registries
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.iot import flags
from googlecloudsdk.command_lib.iot import resource_args
from googlecloudsdk.command_lib.iot import util
from googlecloudsdk.core import log


class Create(base.CreateCommand):
  """Add a new credential to a registry.

  A registry may have at most 10 credentials.
  """

  detailed_help = {
      'DESCRIPTION':
          '{description}',
      'EXAMPLES':
          """\
          To add a credential located in '/path/to/cert.pem' to a registry, run:

            $ {command} --region=us-central1 --registry=my-registry --path=/path/to/cert.pem
          """,
  }

  @staticmethod
  def Args(parser):
    resource_args.AddRegistryResourceArg(parser,
                                         'for which to create credentials',
                                         positional=False)
    flags.AddDeviceRegistryCredentialFlagsToParser(parser)

  def Run(self, args):
    client = registries.RegistriesClient()

    registry_ref = args.CONCEPTS.registry.Parse()
    new_credential = util.ParseRegistryCredential(args.path,
                                                  messages=client.messages)

    credentials = client.Get(registry_ref).credentials
    credentials.append(new_credential)
    response = client.Patch(registry_ref, credentials=credentials)
    log.CreatedResource(registry_ref.Name(), 'credentials for registry')
    return response
